-- Use comment lines to separate commands
-- Channels
create table CHANNEL (
	id 			INTEGER NOT NULL
				PRIMARY KEY GENERATED ALWAYS AS IDENTITY
				(START WITH 1, INCREMENT BY 1),
	name		VARCHAR(256) NOT NULL UNIQUE
)
-- Virtual
create table VIRTUAL (
	name		VARCHAR(256) NOT NULL PRIMARY KEY,
	query		VARCHAR(1000) NOT NULL,
	limit		INTEGER NOT NULL,
	lucene		SMALLINT CONSTRAINT LUCENE_VL CHECK (lucene IN (0,1))
)
-- Feeds
create table FEED (
	id 			INTEGER NOT NULL 
				PRIMARY KEY GENERATED ALWAYS AS IDENTITY 
                (START WITH 1, INCREMENT BY 1),
	name		VARCHAR(256) NOT NULL,
	url			VARCHAR(256) NOT NULL UNIQUE,
	channel		INTEGER NOT NULL,
	lastsync		TIMESTAMP,
	updateperiod	BIGINT NOT NULL,
	icon		BLOB(1M),
	CONSTRAINT CHANNEL_FK FOREIGN KEY (channel) REFERENCES CHANNEL (id)
)
-- Articles in a feed
create table ARTICLE (
    title		VARCHAR(256) NOT NULL,
    authors		VARCHAR(256),
	url			VARCHAR(256),
	feed		INTEGER NOT NULL,
    readflag    SMALLINT CONSTRAINT READ_VL CHECK (readflag IN (0,1)),
    star 		SMALLINT CONSTRAINT STAR_VL CHECK (star IN (0,1)),
	date		TIMESTAMP NOT NULL,
	committed	TIMESTAMP NOT NULL,
	summary		LONG VARCHAR,
	longitude 	DOUBLE,
	latitude 	DOUBLE,

	CONSTRAINT ARTICLE_PK PRIMARY KEY (title, feed, date),
    CONSTRAINT FEED_FK FOREIGN KEY (feed) REFERENCES FEED (id) ON DELETE CASCADE
)
-- Initialization
insert into CHANNEL (name)
values ('')
--
insert into VIRTUAL
values ('Unread', 'select title, authors, url, feed, readflag, star, date, latitude, longitude from article where readflag=0', 0, 0)
--
insert into VIRTUAL
values ('Starred', 'select title, authors, url, feed, readflag, star, date, latitude, longitude from article where star=1', 0, 0)
--
insert into FEED (name, url, channel, lastsync, updateperiod, icon)
values ('Outbox', 'rss-reader://outbox' , 1, null, 10000, null)
--
insert into CHANNEL (name) 
values ('Samples')
-- 
insert into FEED (name, url, channel, updateperiod)
values ('CNN', 'http://rss.cnn.com/rss/cnn_topstories.rss', 2, 100000)
--
insert into FEED (name, url, channel, updateperiod)
values ('Google', 'http://googleblog.blogspot.com/atom.xml', 2, 500000)
-- 
insert into FEED (name, url, channel, updateperiod)
values ('Slashdot', 'http://rss.slashdot.org/Slashdot/slashdot', 2, 350000)
--
insert into FEED (name, url, channel, updateperiod)
values ('BBC', 'http://newsrss.bbc.co.uk/rss/newsonline_world_edition/front_page/rss.xml', 2, 100000)
--
insert into FEED (name, url, channel, updateperiod)
values ('Earthquakes', 'http://earthquake.usgs.gov/eqcenter/recenteqsww/catalogs/eqs7day-M2.5.xml', 2, 100000)
