package controller;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;

/**
 * Prototype dialog for a new profile.
 * @author Kuat
 *
 */
public class NewProfileDialog extends Dialog {
	Controller controller;
	ControllerSettings value;		
	String name;
	Class view;
	int days;
	boolean purgeExpired = true;
	
	/**
	 * Creates a new dialog with given parent shell.
	 * @param parent parent shell
	 * @param original 
	 * 
	 */
	public NewProfileDialog(final Shell parent, Controller controller) {
	    super(parent);
	    this.controller = controller;
	}
	
	/**
	 * Creates a new profile.
	 * @param original initial settings
	 * @return a new profile or null if operation is abandoned.
	 */
	public ControllerSettings open(ControllerSettings original) {
		final Shell shell =
		      new Shell(getParent(), SWT.TITLE | SWT.BORDER | SWT.APPLICATION_MODAL);
		
		shell.setText("Profile properties");
		shell.setLayout(new GridLayout(1,false));
		Composite input = new Composite(shell, SWT.NONE);
		input.setLayoutData(new GridData(GridData.FILL_BOTH));
		input.setLayout(new GridLayout(3,false));
		Composite buttons = new Composite(shell, SWT.NONE);
		buttons.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		buttons.setLayout(new GridLayout(2, false));

		// FIELDS
		Label label = new Label(input,SWT.NONE);
		label.setText("User name:");
		final Text username = new Text(input, SWT.BORDER | SWT.SINGLE);
	    username.addListener(SWT.Modify, new Listener() {
	        public void handleEvent(Event event) {
	           name = username.getText();
	        }
	      });
		GridData gd = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		gd.horizontalSpan = 2;
		username.setLayoutData(gd);

	    
	    final Button purge = new Button(input, SWT.CHECK);
	    purge.setText("Purge un-starred articles after");
		final Spinner interval = new Spinner(input, SWT.BORDER);
		interval.setMaximum(14 * 20);
		interval.setMinimum(14);
		interval.setIncrement(14);
		new Label(input, SWT.NONE).setText("days");
		
		// Set default settings
		view = original.view;
		username.setText(original.userName);
	    this.purgeExpired = (original.purgePeriod != 0);
	    purge.setSelection(this.purgeExpired);
		interval.setEnabled(this.purgeExpired);
		if (this.purgeExpired)
			days = (int) (original.purgePeriod / (60000L*24*60));
		else 
			days = 28;
		interval.setSelection(days);

	    purge.addListener(SWT.Selection, new Listener() {
	    	public void handleEvent(Event e) {
	    		boolean current = purge.getSelection();
	    		purge.setSelection(current);
	    		purgeExpired = current;
	    		interval.setEnabled(current);	    		
	    	}
	    });
	    
	    interval.addListener(SWT.Modify, new Listener() {
	        public void handleEvent(Event event) {
		           days = interval.getSelection();
	        }
	    });
	    		
		// BUTTONS		
		Button button = new Button(buttons, SWT.PUSH);
		button.setText("OK");
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				if ((name != null) && (name.length() > 0)) {
					long period;
					if (!purgeExpired)
						period = 0;
					else
						period = 60000L*24*60*(new Long(days));					
					value = new ControllerSettings(name, period, view, true);
					shell.close();
				} else
					controller.reportError("Wrong profile name");				
				
			}
		});	
		button = new Button(buttons, SWT.PUSH);
		button.setText("Cancel");
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				value = null;
				shell.close();
			}
		});
		
		shell.pack();
	    Monitor primary = shell.getDisplay().getPrimaryMonitor();
	    Rectangle bounds = primary.getBounds();
	    Rectangle rect = shell.getBounds();
	    int x = bounds.x + (bounds.width - rect.width) / 2;
	    int y = bounds.y + (bounds.height - rect.height) / 2;
	    shell.setLocation(x, y);
		shell.open();
		while (!shell.isDisposed()) {
			if (!shell.getDisplay().readAndDispatch()) {
				shell.getDisplay().sleep();
			}
		}

		if ((original != null) && (value != null))
			value.setProfileLocation(original.getProfileLocation());
		return value;
	}
}

