package data;

import java.io.Serializable;

/**
 * Serializable article information suitable for DND.
 * @author oleg
 */
public class DNDArticle implements Serializable {
	private static final long serialVersionUID = -127955363413800734L;
	private final String title;
	private final String parentURL;
	private final long publishedDate;

	private DNDArticle(String title, String parentURL, long publishedDate) {
		this.title = title;
		this.parentURL = parentURL;
		this.publishedDate = publishedDate;
	}
	
	public static DNDArticle fromArticleInfo(ArticleInfo info) {
		if (info == null) return null;
		return new DNDArticle(info.getTitle(), info.getFeed().getURL(), info.getPublishedDate());
	}
	
	/**
	 * Tries to retrieve an article with the same data from the data manager.
	 * @param info serializable info to convert 
	 * @param dataManager the data manager to use
	 * @return an ArticleInfo with the same data or <code>null</code> if <code>info</code> is <code>null</code> or not found 
	 */
	public static ArticleInfo toArticleInfo(DNDArticle info, DataManager dataManager) {
		if (info == null) return null;
		return dataManager.getArticle(info.title, dataManager.getFeed(info.parentURL), info.publishedDate);
	}
	/**
	 * Converts to ArticleInfo and sets its parent feed.
	 * @param info serializable info to convert 
	 * @param dataManager the data manager to use
	 * @param parent new parent feed
	 * @return an ArticleInfo with the same data except the parent feed or <code>null</code> either if <code>info</code> is <code>null</code> or not found
	 */
	public static ArticleInfo toArticleInfo(DNDArticle info, DataManager dataManager, Feed parent) {
		if (info == null) return null;
		ArticleInfo original = toArticleInfo(info, dataManager);
		if (original == null) return null;
		else {
			ArticleInfo newInfo = new ArticleInfo(original.getTitle(),parent,original.getPublishedDate(),original.getAuthors(),original.getURL(),original.isRead(),original.isStarred());
			newInfo.setPoint(original.getPoint());
			return newInfo;
		}
	}
}
