package data;

import java.sql.SQLException;

import util.Trace;

/**
 * Data management exception.
 * @author Kuat
 *
 */
public class DataException extends Exception {

	private static final long serialVersionUID = -604311706660428802L;
	static enum Cause {
		UNKNOWN,
		NULL_POINTER,
		UNIQUENESS,
		MISSING,
		MISSING_PARENT,
		REFERENCE;
	}
	
	private String message;
	private Cause cause = Cause.UNKNOWN;
	
	public DataException(String message){
		this.message = message; 
		Trace.printVerbose(message);
	}
	
	public DataException(SQLException e) {
		this.message = e.getMessage();
		if (e.getSQLState().equals("23505")) {
			this.cause = Cause.UNIQUENESS;
		} else if (e.getSQLState().equals("23503")) {
			this.cause = Cause.REFERENCE;
		}
		Trace.printVerbose(e.getSQLState() + ": " + toString());
	}
	
	public DataException(Object o, Cause cause) {
		this.cause = cause;
		
		if (cause == Cause.UNIQUENESS) {
			this.message = "Duplicate element: " + o.toString();
		} else if (cause == Cause.NULL_POINTER) {
			this.message = "Null pointer exception";
		} else if (cause == Cause.MISSING_PARENT) {
			this.message = "Missing parent reference " + o.toString();
		} else if (cause == Cause.MISSING) {
			this.message = "Missing elements " + o.toString();
		}
		
		Trace.printVerbose(toString());
	}
		
	public boolean isUniquenessViolation() {
		return cause.equals(Cause.UNIQUENESS);
	}
	
	public boolean isNullPointer() {
		return cause.equals(Cause.NULL_POINTER);
	}
	
	public boolean isReferenceViolation() {
		return cause.equals(Cause.REFERENCE);
	}
		
	
	@Override
	public String toString(){
		return message;
	}
	
	@Override
	public String getMessage() {
		return toString();
	}
}
