package data;

import org.eclipse.swt.graphics.ImageData;

/**
 * Wrapper class for a feed.
 * Feeds are uniquely identified by their URL.
 * @author Kuat
 *
 */
public class Feed implements Container{
	private static String userProtocol = "rss-reader://";
		
	private String name;
	private String url;
	private Channel channel;
	private ImageData icon;
	private long lastSynchronized;
	private long updateInterval;

	/** 
	 * Returns the user specified name. Cannot be null.
	 * @return
	 */
	public String getName() {
		return name;
	}

	/**
	 * Feed URL (.rss file for example).
	 * UID of the class.
	 * Can not be null.
	 * For outbox folder it is "rss-reader://outbox".
	 * @return
	 */
	public String getURL() {
		return url;
	}
	
	/**
	 * Retrieves last time of synchronization.
	 * @return
	 */
	public long getSyncTime() {
		return lastSynchronized;
	}
	
	/**
	 * Returns the associated icon image. Can be null.
	 * @return
	 */
	public ImageData getIcon() {
		return icon;
	}
	
	/**
	 * Returns the unique channel parent.
	 * @return
	 */
	public Channel getChannel() {
		return channel;
	}
	
	/**
	 * Returns the update interval in milliseconds.
	 * @return
	 */
	public long getUpdateInterval() {
		return updateInterval;
	}
	
	/**
	 * Indicates whether feed is user-defined (such as outbox)
	 * @return
	 */
	public boolean isLocal() {
		return url.startsWith(userProtocol);
	}
	
	public Feed(String name, String url, long lastModified, Channel channel, long update, ImageData icon) {
		assert channel != null;
		assert name != null;
		assert url != null;
		assert update > 0;
		
		this.name = name;
		this.url = url;
		this.lastSynchronized = lastModified;
		this.channel = channel;
		this.updateInterval = update;
		this.icon = icon;
	}

	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof Feed)) {
			return false;
		}
		if (((Feed) obj).getURL() == null) {
			return url == null;
		} else
			return ((Feed) obj).getURL().equals(url);
	}

	@Override
	public int hashCode() {
		return (url == null ? 0 : url.hashCode());
	}

	@Override
	public String toString() {
		return channel.getName() + " : " + name;
	}
	
	/**
	 * Sets time of the last synchronization in milliseconds.
	 * @param time
	 */
	public void setSyncTime(long time) {
		lastSynchronized = time;
	}
	
	/** Sets update interval in milliseconds.
	 * @param update
	 */
	void setUpdateInterval(long update){
		this.updateInterval = update;
	}

	void setName(String newName) {
		this.name = newName;
	}
	
	void setChannel(Channel channel) {
		this.channel = channel;		
	}
	

}
