package data;

/**
 * Virtual folder representing a dynamic content folder based on 
 * search criterion.
 * Name uniquely identifies the virtual folder.
 * @author Kuat
 *
 */
public class Virtual implements Container{
	private String name;	
	private String query;
	private int limit;
	private boolean isFullText;
	
	/**
	 * Constructs a virtual container of articles.
	 * @param name
	 * @param query
	 */
	Virtual(String name, String query) {
		this.name = name;
		this.query = query;
		this.limit = 0;
		this.isFullText = false;
	}
	
	/**
	 * Constructs a virtual folder.
	 * @param name 
	 * @param query
	 * @param limit maximum number of entries to be shown (zero to show all)
	 */
	Virtual(String name, String query, int limit) {
		this(name, query);
		this.limit = limit;
	}
	
	/**
	 * Constructs a virtual folder with full-text indexer.
	 * @param name
	 * @param query
	 * @param limit
	 * @param fullText
	 */
	Virtual(String name, String query, int limit, boolean fullText) {
		this(name, query, limit);
		this.isFullText = fullText;
	}
	
	/**
	 * Returns the query string used to do search.
	 * @return
	 */
	String getQuery(){
		return query;
	}
	
	/**
	 * Returns the name of the object which uniquely identifies it.
	 * @return
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * Returns the maximum of articles to search or 0 if no limit is specified. 
	 * @return
	 */
	int getLimit() {
		return limit;
	}
	
	/**
	 * Returns true if the query uses full-text indexer.
	 * @return
	 */
	public boolean isFullText() {
		return isFullText;
	}
	
	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof Virtual))
			return false;
		
		return obj.toString().equals(name);
	}

	@Override
	public int hashCode() {
		return name.hashCode();
	}

	@Override
	public String toString() {
		return name;
	}
}
