package settings;

import java.io.Serializable;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;

import util.ArticleCollator;
import data.DNDArticle;

/**
 * Settings for the calendar view.
 * @author oleg
 */
public class CalendarViewSettings implements Serializable {
	private static final long serialVersionUID = 8865920618845386018L;
	public final int mailPaneHeight;
	public final int mailPaneWidth;
	public final int[] mainWeights;
	public final int mailPaneOrientation;
	public final Map<Integer, int[]> weights;
	public final Point shellLocation;
	public final int sortCriterion;
	public final int sortDirection;
	public final int year;
	public final int month;
	public final int day;
	public final DNDArticle mailPaneSelection;
	public final Map<Integer, Integer> columnWidthData;
	public final MailPaneAppearance appearance;
	public CalendarViewSettings(final int mailPaneHeight, final int mailPaneWidth, final int[] mainWeights, final int mailPaneOrientation, final Map<Integer, int[]> weights, final Point shellLocation, final int sortCriterion, final int sortDirection, final int year, final int month, final int day, final DNDArticle mailPaneSelection, final Map<Integer, Integer> columnWidthData, final MailPaneAppearance appearance) {
		this.mailPaneHeight = mailPaneHeight;
		this.mailPaneWidth = mailPaneWidth;
		this.mainWeights = mainWeights;
		this.mailPaneOrientation = mailPaneOrientation;
		this.weights = weights;
		this.shellLocation = shellLocation;
		this.sortCriterion = sortCriterion;
		this.sortDirection = sortDirection;
		this.year = year;
		this.month = month;
		this.day = day;
		this.mailPaneSelection = mailPaneSelection;
		this.columnWidthData = columnWidthData;
		this.appearance = appearance;
	}
	/**
	 * Returns default calendar view settings.
	 * @return
	 */
	public static CalendarViewSettings defaultSettings() {
		Map<Integer, int[]> weights = new HashMap<Integer, int[]>();
		weights.put(SWT.VERTICAL, new int[]{1,1});
		weights.put(SWT.HORIZONTAL, new int[]{1,1});
		Map<Integer, Integer> columnData = new HashMap<Integer, Integer>();
		columnData.put(ArticleCollator.SORT_DATE, 100);
		columnData.put(ArticleCollator.SORT_FEED, 100);
		columnData.put(ArticleCollator.SORT_READ, 20);
		columnData.put(ArticleCollator.SORT_STAR, 20);
		Calendar c = new GregorianCalendar();
		return new CalendarViewSettings(500,600,
				new int[]{1,2},SWT.VERTICAL,weights,
				new Point(0,0),ArticleCollator.SORT_DATE, 
				SWT.UP,c.get(Calendar.YEAR),c.get(Calendar.MONTH), 
				c.get(Calendar.DAY_OF_MONTH), null, columnData, 
				MailPaneAppearance.defaultAppearance());
	}
}
