package settings;

import java.io.Serializable;

import org.eclipse.swt.SWT;

import util.CSSUtils;
import util.SFontData;
import util.SRGB;

/**
 * The style of an article displayed in the mail view.  
 * @author oleg
 */
public class MailPaneAppearance implements Serializable {
	private static final long serialVersionUID = -5413263723451351356L;
	/**
	 * Generates CSS for displaying the summary of a folder.
	 * @return CSS specifying the same appearance
	 */
	public String getFolderCSS() {
		StringBuffer css = new StringBuffer("body {overflow: auto; font: ");
		css.append(CSSUtils.getCSSFont(articleFont)).append("; color: ");
		css.append(CSSUtils.getCSSColor(articleColor)).append("; background-color: ");
		css.append(CSSUtils.getCSSColor(articleBackground));
		css.append("}\ndiv.folderDiv {font: ").append(CSSUtils.getCSSFont(titleFont));
		css.append("; color: ").append(CSSUtils.getCSSColor(titleColor));
		css.append("; background-color: ").append(CSSUtils.getCSSColor(headerBackground));
		css.append("; padding: 5px; text-align: center");
		return css.append("}").toString();
	}
	
	/**
	 * Generates CSS for displaying an article.
	 * @return the CSS specifying the same appearance
	 */
	public String getArticleCSS() {
		StringBuffer css = new StringBuffer("body {overflow: auto; font: ");
		css.append(CSSUtils.getCSSFont(articleFont)).append("; color: ");
		css.append(CSSUtils.getCSSColor(articleColor)).append("; background-color: ");
		css.append(CSSUtils.getCSSColor(articleBackground)).append("}\np.rssheader {font: ");
		css.append(CSSUtils.getCSSFont(dateFont)).append("; color: ");
		css.append(CSSUtils.getCSSColor(dateColor)).append("; background-color: ");
		css.append(CSSUtils.getCSSColor(headerBackground)).append("; padding: 10px}");
		css.append("\na.rsslink {font: ").append(CSSUtils.getCSSFont(titleFont)).append("; color: ");
		css.append(CSSUtils.getCSSColor(titleColor)).append("; text-decoration:none}");
		return css.toString();
	}
	SFontData titleFont;
	SRGB titleColor;
	SFontData dateFont;
	SRGB dateColor;
	SRGB headerBackground;
	SFontData articleFont;
	SRGB articleColor;
	SRGB articleBackground;
	/**
	 * Constructor.
	 * @param titleFont
	 * @param titleColor
	 * @param dateFont
	 * @param dateColor
	 * @param headerBackground
	 * @param articleFont
	 * @param articleColor
	 * @param articleBackground
	 */
	public MailPaneAppearance(final SFontData titleFont, final SRGB titleColor, final SFontData dateFont, final SRGB dateColor, final SRGB headerBackground, final SFontData articleFont, final SRGB articleColor, final SRGB articleBackground) {
		this.titleFont = titleFont;
		this.titleColor = titleColor;
		this.dateFont = dateFont;
		this.dateColor = dateColor;
		this.headerBackground = headerBackground;
		this.articleFont = articleFont;
		this.articleColor = articleColor;
		this.articleBackground = articleBackground;
	}
	/**
	 * Copy constructor.
	 * @param appearance
	 */
	public MailPaneAppearance(MailPaneAppearance appearance) {
		this.titleFont = appearance.titleFont;
		this.titleColor = appearance.titleColor;
		this.dateFont = appearance.dateFont;
		this.dateColor = appearance.dateColor;
		this.headerBackground = appearance.headerBackground;
		this.articleFont = appearance.articleFont;
		this.articleColor = appearance.articleColor;
		this.articleBackground = appearance.articleBackground;
	}
	
	public static MailPaneAppearance defaultAppearance() {
		return new MailPaneAppearance(
				new SFontData("Trebuchet MS",14, SWT.NORMAL),
				new SRGB(255,255,255),
				new SFontData("Tahoma",8,SWT.NORMAL),
				new SRGB(0xff,0xff,0xff),
				new SRGB(0x34,0x65,0xa4),
				new SFontData("Tahoma",8,SWT.NORMAL),
				new SRGB(0,0,0),
				new SRGB(0xff,0xff,0xff));
	}
}
