package settings;

import java.io.Serializable;

import org.eclipse.swt.SWT;

import util.CSSUtils;
import util.SFontData;
import util.SRGB;

/**
 * Newspaper view style.
 * @author oleg
 */
public class NewspaperPaneAppearance implements Serializable {
	private static final long serialVersionUID = -2443223166929162802L;
	SFontData headerFont;
	SRGB headerColor;
	SFontData titleFont;
	SRGB titleColor;
	SFontData authorFont;
	SRGB authorColor;
	SFontData articleFont;
	SRGB articleColor;
	SRGB backgroundColor;
	public SRGB getArticleColor() {
		return articleColor;
	}
	public SFontData getArticleFont() {
		return articleFont;
	}
	public SRGB getAuthorColor() {
		return authorColor;
	}
	public SFontData getAuthorFont() {
		return authorFont;
	}
	public SRGB getBackgroundColor() {
		return backgroundColor;
	}
	public SRGB getHeaderColor() {
		return headerColor;
	}
	public SFontData getHeaderFont() {
		return headerFont;
	}
	public SRGB getTitleColor() {
		return titleColor;
	}
	public SFontData getTitleFont() {
		return titleFont;
	}
	public NewspaperPaneAppearance(SFontData headerFont, SRGB headerColor, SFontData titleFont, SRGB titleColor, SFontData authorFont, SRGB authorColor, SFontData articleFont, SRGB articleColor, SRGB backgroundColor) {
		this.headerFont = headerFont;
		this.headerColor = headerColor;
		this.titleFont = titleFont;
		this.titleColor = titleColor;
		this.authorFont = authorFont;
		this.authorColor = authorColor;
		this.articleFont = articleFont;
		this.articleColor = articleColor;
		this.backgroundColor = backgroundColor;
	}
	public NewspaperPaneAppearance(NewspaperPaneAppearance appearance) {
		this.headerFont = appearance.headerFont;
		this.headerColor = appearance.headerColor;
		this.titleFont = appearance.titleFont;
		this.titleColor = appearance.titleColor;
		this.authorFont = appearance.authorFont;
		this.authorColor = appearance.authorColor;
		this.articleFont = appearance.articleFont;
		this.articleColor = appearance.articleColor;
		this.backgroundColor = appearance.backgroundColor;
	}
	/**
	 * Generates CSS for the browser in the newspaper view.
	 * @return CSS style corresponding to the object state
	 */
	public String getCSS() {
		StringBuffer css = new StringBuffer("body {overflow: auto; background-color: ");
		css.append(CSSUtils.getCSSColor(backgroundColor)).append("}\n");
		css.append("div.rss-column {position:absolute;width:28%;padding:1%}\n");
		css.append("a.rss-header {font: ").append(CSSUtils.getCSSFont(titleFont));
		css.append("; color: ").append(CSSUtils.getCSSColor(titleColor));
		css.append("; text-decoration: none}\n");
		css.append("div.rss-author {font: ").append(CSSUtils.getCSSFont(authorFont));
		css.append("; color: ").append(CSSUtils.getCSSColor(authorColor)).append("}\n");
		css.append("div.rss-article {font: ").append(CSSUtils.getCSSFont(articleFont));
		css.append("; color: ").append(CSSUtils.getCSSColor(articleColor)).append("; overflow: auto}\n");
		css.append("div.rss-main {text-align: center; border-bottom: black 3px solid; padding: 4px; font: ");
		css.append(CSSUtils.getCSSFont(headerFont)).append("; color: ");
		css.append(CSSUtils.getCSSColor(headerColor)).append("}\n");
		return css.toString();
	}
	/**
	 * Returns default newspaper view appearance.
	 * @return
	 */
	public static NewspaperPaneAppearance defaultAppearance() {
		return new NewspaperPaneAppearance(new SFontData("Trebuchet MS", 14, SWT.BOLD), 
				new SRGB(0,0,0), 
				new SFontData("Tahoma", 11, SWT.BOLD), 
				new SRGB(0x20,0x4a,0x87), 
				new SFontData("Tahoma", 9, SWT.NORMAL), 
				new SRGB(0xba,0xbd,0xb6), 
				new SFontData("Tahoma", 8, SWT.NORMAL), 
				new SRGB(0,0,0),
				new SRGB(0xff,0xff,0xff));
	}
}
