package settings;

import org.eclipse.swt.widgets.Shell;

import view.components.NewspaperPane;

/**
 * A dialog for editing the style of the newspaper view.
 * @author oleg
 */
public class NewspaperPaneAppearanceDialog extends AppearanceDialog {
	@Override
	protected void showAppearance() {
		StringBuffer xhtml = new StringBuffer("<html><head><title></title><style type=\"text/css\">\n");
		xhtml.append(appearance.getCSS()).append("\n</style></head><body>");
		xhtml.append("<div class=\"rss-main\">Feed Name</div>");
		xhtml.append("<div><a class=\"rss-header\" href=\"\">Article</a>\n");
		xhtml.append("<div class=\"rss-author\">By Author</div>\n");
		xhtml.append("<div class=\"rss-article\">contents</div></div>");
		xhtml.append("</body></html>");
		browser.setText(xhtml.toString());
	}
	private final NewspaperPane pane;
	private NewspaperPaneAppearance appearance;
	/**
	 * Create and show a new dialog.
	 * @param parent parent shell
	 * @param currentAppearance the current style
	 * @param view the mail view to return a new style to 
	 */
	public NewspaperPaneAppearanceDialog(Shell parent, final NewspaperPane pane) {
		super(parent, 9);
		this.appearance = new NewspaperPaneAppearance(pane.getAppearance());
		this.pane = pane;
		addFontButton("Change header font...", "Select header font", appearance, "headerFont", "headerColor");
		addColorButton("Change header font color...", "Select header font color", appearance, "headerColor");		
		addFontButton("Change title font...", "Select title font", appearance, "titleFont", "titleColor");
		addColorButton("Change title font color...", "Select title font color", appearance, "titleColor");
		addFontButton("Change author font...", "Select author font", appearance, "authorFont", "authorColor");
		addColorButton("Change author font color...", "Select author font color", appearance, "authorColor");
		addFontButton("Change article font...", "Select article font", appearance, "articleFont", "articleColor");
		addColorButton("Change article font color...", "Select article font color", appearance, "articleColor");
		addColorButton("Change background color...", "Select background color", appearance, "backgroundColor");
		
		showAppearance();
		
		shell.pack();
		shell.open();
		
		while (!input.isDisposed()) {
			if (!input.getDisplay().readAndDispatch()) {
				input.getDisplay().sleep();
			}
		}
	}
	@Override
	protected void applyClicked() {
		pane.setAppearance(appearance);
	}
}
