package test;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import parser.RSSParser;
import util.Pair;

import com.sun.syndication.io.FeedException;

import data.ArticleContent;
import data.ArticleInfo;
import data.Feed;

/**
 * Tests the parser for correctness with RSS 0.91, 0.92, 1.0, 2.0 and ATOM 2.0 samples.
 * @author oleg
 */

public class RSSParserTest extends TestCase{

	final Feed fakeFeed = new Feed("foo","bar",0,null,0,null);
	
	final Comparator<Pair<ArticleInfo, ArticleContent>> cmp = new Comparator<Pair<ArticleInfo,ArticleContent>>() {

		public int compare(Pair<ArticleInfo, ArticleContent> o1, Pair<ArticleInfo, ArticleContent> o2) {
			return o1.car.getTitle().compareTo(o2.car.getTitle());
		}
		
	};
	
	public RSSParserTest(String name){
		super(name);
	}
	
	public void atom10test() {
		try {
			Collection<Pair<ArticleInfo, ArticleContent>> collection = RSSParser.parseFeed(fakeFeed,new FileInputStream("test/atom10"),1);
			List<Pair<ArticleInfo, ArticleContent>> lst = new LinkedList<Pair<ArticleInfo,ArticleContent>>();
			lst.addAll(collection);
			Collections.sort(lst, cmp);
			ArticleInfo info = lst.get(0).car;
			ArticleContent cont = lst.get(0).cdr;
			assertEquals("Title is wrong", info.getTitle(), "Atom draft-07 snapshot");
			assertEquals("URL is wrong", info.getURL(), "http://example.org/2005/04/02/atom");
			assertTrue("Content is wrong", cont.getSummary().contains("<p><i>[Update: The Atom draft is finished.]</i></p>"));
			assertEquals("Date is wrong", info.getPublishedDate(), 1071318569000L);
			assertEquals("Authors are wrong", info.getAuthors(), "Mark Pilgrim"); 	
		} catch (IOException e) {
			assertFalse("IOException caught",true);
		} catch (FeedException e) {
			assertFalse("FeedException caught", true);
		}
	}
	
	public void rss91test() {
		try {
			Collection<Pair<ArticleInfo, ArticleContent>> collection = RSSParser.parseFeed(fakeFeed,new FileInputStream("test/rss91"),1);
			List<Pair<ArticleInfo, ArticleContent>> lst = new LinkedList<Pair<ArticleInfo,ArticleContent>>();
			lst.addAll(collection);
			Collections.sort(lst, cmp);
			ArticleInfo info = lst.get(0).car;
			ArticleContent cont = lst.get(0).cdr;
			assertEquals("Title is wrong", info.getTitle(), "Giving the world a pluggable Gnutella");
			assertEquals("URL is wrong", info.getURL(), "http://writetheweb.com/read.php?item=24");
			assertEquals("Content is wrong", cont.getSummary(),"WorldOS is a framework on which to build programs that work like Freenet or Gnutella -allowing distributed applications using peer-to-peer routing.");
			assertEquals("Date is wrong", info.getPublishedDate(), 1);
			assertEquals("Authors are wrong", info.getAuthors(), ""); 	
		} catch (IOException e) {
			assertFalse("IOException caught",true);
		} catch (FeedException e) {
			assertFalse("FeedException caught", true);
		}
	}
	
	public void rss92test() {
		try {
			Collection<Pair<ArticleInfo, ArticleContent>> collection = RSSParser.parseFeed(fakeFeed,new FileInputStream("test/rss92"),1);
			List<Pair<ArticleInfo, ArticleContent>> lst = new LinkedList<Pair<ArticleInfo,ArticleContent>>();
			lst.addAll(collection);
			Collections.sort(lst, cmp);
			ArticleInfo info = lst.get(0).car;
			ArticleContent cont = lst.get(0).cdr;
			assertEquals("Content is wrong", cont.getSummary(),"It's been a few days since I added a song to the Grateful Dead channel. Now that there are all these new Radio users, many of whom are tuned into this channel (it's #16 on the hotlist of upstreaming Radio users, there's no way of knowing how many non-upstreaming users are subscribing, have to do something about this..). Anyway, tonight's song is a live version of Weather Report Suite from Dick's Picks Volume 7. It's wistful music. Of course a beautiful song, oft-quoted here on Scripting News. <i>A little change, the wind and rain.</i>\n");
			assertEquals("Date is wrong", info.getPublishedDate(), 1); 	
		} catch (IOException e) {
			assertFalse("IOException caught",true);
		} catch (FeedException e) {
			assertFalse("FeedException caught", true);
		}
	}

	public void rss10test() {
		try {
			Collection<Pair<ArticleInfo, ArticleContent>> collection = RSSParser.parseFeed(fakeFeed,new FileInputStream("test/rss10"),1);
			List<Pair<ArticleInfo, ArticleContent>> lst = new LinkedList<Pair<ArticleInfo,ArticleContent>>();
			lst.addAll(collection);
			Collections.sort(lst, cmp);
			ArticleInfo info = lst.get(0).car;
			ArticleContent cont = lst.get(0).cdr;
			assertEquals("Title is wrong", info.getTitle(), "Processing Inclusions with XSLT");
			assertEquals("URL is wrong", info.getURL(), "http://xml.com/pub/2000/08/09/xslt/xslt.html");
			assertTrue("Content is wrong", cont.getSummary().contains("Processing document inclusions with general XML"));
			assertEquals("Date is wrong", info.getPublishedDate(), 1); 	
		} catch (IOException e) {
			assertFalse("IOException caught",true);
		} catch (FeedException e) {
			assertFalse("FeedException caught", true);
		}	
	}

	public void rss20test() {
		try {
			Collection<Pair<ArticleInfo, ArticleContent>> collection = RSSParser.parseFeed(fakeFeed,new FileInputStream("test/rss20"),1);
			List<Pair<ArticleInfo, ArticleContent>> lst = new LinkedList<Pair<ArticleInfo,ArticleContent>>();
			lst.addAll(collection);
			Collections.sort(lst, cmp);
			ArticleInfo info = lst.get(0).car;
			ArticleContent cont = lst.get(0).cdr;
			assertEquals("Title is wrong", info.getTitle(), "Astronauts' Dirty Laundry");
			assertEquals("URL is wrong", info.getURL(), "http://liftoff.msfc.nasa.gov/news/2003/news-laundry.asp");
			assertEquals("Content", cont.getSummary(),"Compared to earlier spacecraft, the International Space Station has many luxuries, but laundry facilities are not one of them.  Instead, astronauts have other options.");
			assertEquals("Date is wrong", info.getPublishedDate(), 1053420962000L); 	
		} catch (IOException e) {
			assertFalse("IOException caught",true);
		} catch (FeedException e) {
			assertFalse("FeedException caught", true);
		}	
	}

	
	
	public static Test suite(){
		TestSuite suite = new TestSuite();
		
		suite.addTest(new RSSParserTest("rss91test"));
		suite.addTest(new RSSParserTest("rss92test"));
		suite.addTest(new RSSParserTest("rss10test"));
		suite.addTest(new RSSParserTest("rss20test"));
		suite.addTest(new RSSParserTest("atom10test"));
		
	return suite;

	}
}
