package test;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import util.SortUtils;

/**
 * Test suite for the sorting utils.
 * @author oleg
 */
public class SortUtilTest extends TestCase {
	
	public SortUtilTest(String s) {
		super(s);
	}
	
	public static final Comparator<Integer> intCompare = new Comparator<Integer>() {
		public int compare(Integer i1, Integer i2) {
			return i1 - i2;
		}
	};
	
	public void addToSortedTest() {
		List<Integer> sorted = new ArrayList<Integer>();
		List<Integer> correct = new LinkedList<Integer>();
		for (int i = 0; i< 10; i++) {
			sorted.add(2*i);
		}
		List<Integer> toadd = new LinkedList<Integer>();
		toadd.add(15);
		toadd.add(35);
		toadd.add(1);
		toadd.add(7);
		correct.addAll(toadd);
		correct.addAll(sorted);
		Collections.sort(correct, intCompare);
		SortUtils.addToSorted(sorted, Collections.unmodifiableCollection(toadd), intCompare);
		assertEquals("The order is incorrect", sorted, correct); 
	}
	
	public void removeAllTest() {
		List<Integer> list = new ArrayList<Integer>();
		for (int i=0; i< 10; i++) {
			list.add(i);
		}
		List<Integer> list2 = new LinkedList<Integer>();
		list2.addAll(list);
		int[] indices = new int[] {4,2,5};
		SortUtils.removeAll(list, indices);
		for (int i: indices) list2.remove((Integer) i);
		assertEquals("Wrong elements are deleted",list,list2);
		assertTrue("Indices are not sorted", Arrays.equals(indices, new int[] {2,4,5}));
	}
	
	public void binarySearchTest() {
		Integer[] a = new Integer[] {0,2,4,6};
		assertEquals("Wrong position", SortUtils.binaryPositionSearch(a, 0, 4, 3, intCompare), 2);
		assertEquals("Wrong position", SortUtils.binaryPositionSearch(a, 0, 3, 3, intCompare), 2);
		assertEquals("Wrong position", SortUtils.binaryPositionSearch(a, 0, 2, 3, intCompare), 2);
		assertEquals("Wrong position", SortUtils.binaryPositionSearch(a, 0, 1, 3, intCompare), 1);
		assertEquals("Wrong position", SortUtils.binaryPositionSearch(a, 1, 4, 1, intCompare), 1);
		assertEquals("Wrong position", SortUtils.binaryPositionSearch(a, 0, 3, 5, intCompare), 3);

	}
	
	public static Test suite() {
		TestSuite suite = new TestSuite();
		suite.addTest(new SortUtilTest("addToSortedTest"));
		suite.addTest(new SortUtilTest("removeAllTest"));
		suite.addTest(new SortUtilTest("binarySearchTest"));
		return suite;
	}
	

}
