package util;

import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.swt.SWT;

import data.ArticleInfo;

/**
 * Provides standard comparators for RSS articles.
 * @author oleg
 */
public class ArticleCollator {
	public static final int SORT_TITLE = 0;
	public static final int SORT_DATE = 1;
	public static final int SORT_STAR = 2;
	public static final int SORT_READ = 3;
	public static final int SORT_FEED = 4;
	
	private static final ArticleCollator instance = new ArticleCollator();
	
	public static ArticleCollator instance() {
		return instance;
	}
	
	/**
	 * Gets a standard article comparator.
	 * @param type comparator type (<code>SORT_TITLE</code>, <code>SORT_DATE</code>, <code>SORT_STAR</code>, <code>SORT_READ</code>, or <code>SORT_FEED</code>)
	 * @param direction sort direction (<code>SWT.UP</code> or <code>SWT.DOWN</code>)
	 * @return
	 */
	public Comparator<ArticleInfo> getComparator(int type, int direction) {
		return collators.get(type).get(direction);
	}
	
	private final Map<Integer, Map<Integer,Comparator<ArticleInfo>>> collators;
	
	private ArticleCollator() {
		collators = new HashMap<Integer, Map<Integer,Comparator<ArticleInfo>>>();
		Map<Integer, Comparator<ArticleInfo>> couple = new HashMap<Integer, Comparator<ArticleInfo>>();
		couple.put(SWT.UP, new Comparator<ArticleInfo>() {
			public int compare(ArticleInfo a1, ArticleInfo a2) {
				int tc = a1.getTitle().compareTo(a2.getTitle());
				if (tc == 0) {
					if (a1.getPublishedDate() > a2.getPublishedDate()) return 1;
					else if (a1.getPublishedDate() < a2.getPublishedDate()) return -1;
					else {
						return a1.getFeed().getURL().compareTo(a2.getFeed().getURL());
					}
				} else return -tc;
			}
		});
		couple.put(SWT.DOWN, new Comparator<ArticleInfo>() {
			public int compare(ArticleInfo a1, ArticleInfo a2) {
				int tc = a1.getTitle().compareTo(a2.getTitle());
				if (tc == 0) {
					if (a1.getPublishedDate() > a2.getPublishedDate()) return 1;
					else if (a1.getPublishedDate() < a2.getPublishedDate()) return -1;
					else {
						return a1.getFeed().getURL().compareTo(a2.getFeed().getURL());
					}
				} else return tc;
			}
		});
		collators.put(SORT_TITLE, couple);
		couple = new HashMap<Integer, Comparator<ArticleInfo>>();
		couple.put(SWT.UP, new Comparator<ArticleInfo>() {
			public int compare(ArticleInfo a1, ArticleInfo a2) {
				if (a1.getPublishedDate() > a2.getPublishedDate()) return 1;
				else if (a1.getPublishedDate() < a2.getPublishedDate()) return -1;
				else {
					int tc = a1.getTitle().compareTo(a2.getTitle());
					if (tc == 0) return a1.getFeed().getURL().compareTo(a2.getFeed().getURL());
					else return tc;
				}
			}
		});
		couple.put(SWT.DOWN, new Comparator<ArticleInfo>() {
			public int compare(ArticleInfo a1, ArticleInfo a2) {
				if (a1.getPublishedDate() > a2.getPublishedDate()) return -1;
				else if (a1.getPublishedDate() < a2.getPublishedDate()) return 1;
				else {
					int tc = a1.getTitle().compareTo(a2.getTitle());
					if (tc == 0) return a1.getFeed().getURL().compareTo(a2.getFeed().getURL());
					else return tc;
				}
			}
		});
		collators.put(SORT_DATE, couple);
		couple = new HashMap<Integer, Comparator<ArticleInfo>>();
		couple.put(SWT.UP, new Comparator<ArticleInfo>() {
			public int compare(ArticleInfo a1, ArticleInfo a2) {
				if (a1.isStarred() && !a2.isStarred()) return 1;
				else if (!a1.isStarred() && a2.isStarred()) return -1;
				else {
					if (a1.getPublishedDate() > a2.getPublishedDate()) return 1;
					else if (a1.getPublishedDate() < a2.getPublishedDate()) return -1;
					else {
						int tc = a1.getTitle().compareTo(a2.getTitle());
						if (tc == 0) return a1.getFeed().getURL().compareTo(a2.getFeed().getURL());
						else return tc;
					}
				}
			}
		});
		couple.put(SWT.DOWN, new Comparator<ArticleInfo>() {
			public int compare(ArticleInfo a1, ArticleInfo a2) {
				if (a1.isStarred() && !a2.isStarred()) return -1;
				else if (!a1.isStarred() && a2.isStarred()) return 1;
				else {
					if (a1.getPublishedDate() > a2.getPublishedDate()) return -1;
					else if (a1.getPublishedDate() < a2.getPublishedDate()) return 1;
					else {
						int tc = a1.getTitle().compareTo(a2.getTitle());
						if (tc == 0) return a1.getFeed().getURL().compareTo(a2.getFeed().getURL());
						else return tc;
					}
				}
			}
		});		
		collators.put(SORT_STAR, couple);
		couple = new HashMap<Integer, Comparator<ArticleInfo>>();
		couple.put(SWT.UP, new Comparator<ArticleInfo>() {
			public int compare(ArticleInfo a1, ArticleInfo a2) {
				if (a1.isRead() && !a2.isRead()) return -1;
				else if (!a1.isRead() && a2.isRead()) return 1;
				else {
					if (a1.getPublishedDate() > a2.getPublishedDate()) return 1;
					else if (a1.getPublishedDate() < a2.getPublishedDate()) return -1;
					else {
						int tc = a1.getTitle().compareTo(a2.getTitle());
						if (tc == 0) return a1.getFeed().getURL().compareTo(a2.getFeed().getURL());
						else return tc;
					}
				}
			}
		});
		couple.put(SWT.DOWN, new Comparator<ArticleInfo>() {
			public int compare(ArticleInfo a1, ArticleInfo a2) {
				if (a1.isRead() && !a2.isRead()) return 1;
				else if (!a1.isRead() && a2.isRead()) return -1;
				else {
					if (a1.getPublishedDate() > a2.getPublishedDate()) return 1;
					else if (a1.getPublishedDate() < a2.getPublishedDate()) return -1;
					else {
						int tc = a1.getTitle().compareTo(a2.getTitle());
						if (tc == 0) return a1.getFeed().getURL().compareTo(a2.getFeed().getURL());
						else return tc;
					}
				}
			}
		});
		collators.put(SORT_READ, couple);
		couple = new HashMap<Integer, Comparator<ArticleInfo>>();
		couple.put(SWT.UP, new Comparator<ArticleInfo>() {
			public int compare(ArticleInfo a1, ArticleInfo a2) {
				int feedcp = a1.getFeed().getName().compareTo(a2.getFeed().getName());
				if (feedcp == 0) {
					int feedUrlCp = a1.getFeed().getURL().compareTo(a2.getFeed().getURL());
					if (feedUrlCp == 0) {
						if (a1.getPublishedDate() > a2.getPublishedDate()) return 1;
						else if (a1.getPublishedDate() < a2.getPublishedDate()) return -1;
						else return a1.getTitle().compareTo(a2.getTitle());
					} else return feedUrlCp;
				} else return -feedcp;
			}
		});
		couple.put(SWT.DOWN, new Comparator<ArticleInfo>() {
			public int compare(ArticleInfo a1, ArticleInfo a2) {
				int feedcp = a1.getFeed().getName().compareTo(a2.getFeed().getName());
				if (feedcp == 0) {
					int feedUrlCp = a1.getFeed().getURL().compareTo(a2.getFeed().getURL());
					if (feedUrlCp == 0) {
						if (a1.getPublishedDate() > a2.getPublishedDate()) return 1;
						else if (a1.getPublishedDate() < a2.getPublishedDate()) return -1;
						else return a1.getTitle().compareTo(a2.getTitle());
					} else return feedUrlCp;
				} else return feedcp;
			}
		});
		collators.put(SORT_FEED, couple);
	}
}
