package util;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;
import org.eclipse.swt.widgets.Display;

/**
 * Loads local icons and web favicons.
 * @author oleg
 */
public class IconFactory {
	public static enum ICON {
		FOLDER ("folder.png"),
		FOLDER_SEARCH ("folder-search.png"),
		FOLDER_OUTBOX ("folder-outbox.png"),
		FOLDER_UNREAD ("folder-unread.png"),
		NEW_FEED ("new-feed.png"),
		NEW_FOLDER ("new-folder.png"),
		UPDATE ("update.png"),
		SEARCH ("search.png"),
		FAVICON ("favicon.png"),
		ONLINE ("online.png"),
		OFFLINE ("offline.png"),
		NEW_ARTICLE ("new-article.png"),
		DELETE ("delete.png"),
		MOVE ("move.png"),
		UNDO ("undo.png"),
		RENAME ("rename.png"),
		SEARCH_TECHNORATI ("technorati.png"),
		SEARCH_FEEDSTER ("feedster.png"),
		SEARCH_SYNDIC8	("syndic8.png"),
		SEARCH_BLOGDIGGER ("blogdigger.png"),
		TRAY_ICON ("tray.png"),
		TRAY_ICON_BUSY ("tray-busy.png"),
		SEARCH_CLEAR ("search-clear.png"),
		STAR ("star.png"),
		UNREAD ("unread.png"),
		SAVE ("save.png"),
		TIME ("time.png"),
		LIVEJOURNAL ("lj.png"),
		EXIT ("exit.png"),
		HELP ("help.png"),
		UNSTAR ("unstar.png"),
		GOOGLE_MAPS ("google-maps.png"),
		WIDGET_VIEW ("widget-view.png"),
		MAIL_VIEW ("mail-view.png"),
		NEWSPAPER_VIEW ("newspaper-view.png"),
		CALENDAR ("calendar-view.png"),
		LAYOUT ("layout.png"),
		WEB_SEARCH ("web-search.png"),
		LIGHT_BULB ("bulb.png"),
		FONTS_AND_COLORS ("fonts.png"),		
		LINK ("link.png"),
		LOGO ("logo.png"),
		PROFILE ("profile.png");
		private final String fileName; 
		private ICON(String fileName) {
			this.fileName = fileName;
		}
	}
	// local cache
	private static Map<ICON,ImageData> icons = new HashMap<ICON,ImageData>(); 
	
	/**
	 * Gets the favorite icon of a website.
	 * @param url the URL
	 * @return the favorite icon or a standard icon
	 */
	public static ImageData getFavicon(String url) {
		try {
			URL ru = new URL(url);
			// this is only semi-correct, but parsing the actual HTML is not worth it
			URL iconurl = new URL(new StringBuffer().append(ru.getProtocol()).append("://").append(ru.getHost()).append("/favicon.ico").toString());
			ImageData[] data = new ImageLoader().load(iconurl.openStream());
			if (data.length == 0) return getIcon(ICON.FAVICON);
			else return data[data.length - 1];
		} catch (Exception e) {
			return getIcon(ICON.FAVICON);
		}
	}
	/**
	 * Gets a standard application icon.
	 * @param icon
	 * @return
	 */
	public static ImageData getIcon(ICON icon) {
		ImageData cache = icons.get(icon);
		if (cache == null) {
			//cache = new ImageData("img/" + icon.fileName);
			// TODO: use this when packing in a jar
			cache = new ImageData(IconFactory.class.getResourceAsStream("/img/" + icon.fileName));
			icons.put(icon,cache);
			return cache;
		} else return cache;
	}
	/**
	 * Createsa standard application icon on a display.
	 * @param display display
	 * @param icon icon id
	 * @return image
	 */
	public static Image getIcon(Display display, ICON icon) {
		return new Image(display,getIcon(icon));
	}
}
