package view;

import java.util.Collection;
import java.util.Iterator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;

import search.SearchBox;
import settings.CalendarViewSettings;
import settings.MailPaneAppearanceDialog;
import view.components.CalendarPane;
import view.components.RSSMailPane;
import view.components.StatusBar;
import controller.Controller;
import controller.StandardMenu;
import data.ArticleInfo;
import data.Channel;
import data.DNDArticle;
import data.Feed;
import data.Virtual;

/**
 * Calendar view. 
 * @author oleg
 *
 */
public class CalendarView extends View {
	private final Menu menuBar;
	private final RSSMailPane mailPane;
	private final SashForm sashForm;
	private final StatusBar statusBar;
	private final CalendarPane calPane;
	private final SearchBox search;
	
	public CalendarView(final Shell parent, final Controller controller) {
		super(parent, controller);
		CalendarViewSettings settings = (CalendarViewSettings) controller.getSettings(this.getClass().getName());
		if (settings == null) {
			settings = CalendarViewSettings.defaultSettings();
		}
		GridLayout calViewLayout = new GridLayout(1,false);
		calViewLayout.horizontalSpacing = 0;
		calViewLayout.marginWidth = 0;
		calViewLayout.marginHeight = 0;
		calViewLayout.verticalSpacing = 0;
		this.setLayout(calViewLayout);
		Composite bar = new Composite(this, SWT.None);		
		bar.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		GridLayout barLayout = new GridLayout(2, false);
		barLayout.verticalSpacing = 0;
		barLayout.marginHeight = 0;
		bar.setLayout(barLayout);
		controller.getActionManager().createStandardToolbar(bar);
		sashForm = new SashForm(this,SWT.HORIZONTAL);
		calPane = new CalendarPane(sashForm);
		mailPane = new RSSMailPane(sashForm, controller,calPane,settings.appearance,settings.sortCriterion,settings.sortDirection,settings.columnWidthData,settings.mailPaneOrientation,settings.weights);
		search = new SearchBox(controller, bar, mailPane, true, false, true);
		calPane.addContainerSelectionObserver(mailPane);
		GridData gData = new GridData(GridData.FILL_BOTH);
		gData.widthHint = settings.mailPaneWidth;
		gData.heightHint = settings.mailPaneHeight;
		sashForm.setLayoutData(gData);
		sashForm.setWeights(settings.mainWeights);
		statusBar = new StatusBar(this, controller, SWT.BORDER);
		statusBar.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		controller.getDownloadManager().addDownloadObserver(statusBar);
		calPane.setSelectedDay(settings.year, settings.month, settings.day);
		mailPane.setSelectedArticle(DNDArticle.toArticleInfo(settings.mailPaneSelection, dataManager));
		
		StandardMenu stMenu = controller.getActionManager().createStandardMenuBar();
		Menu viewMenu = stMenu.getSubMenu("View");
		MenuItem mi = new MenuItem(viewMenu,SWT.CASCADE,0);
		mi.setText("Fonts and colors...");
		mi.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				new MailPaneAppearanceDialog(parent,mailPane);
			}
		});
		
		mi = new MenuItem(viewMenu, SWT.CASCADE,1);
		mi.setText("Layout");
		Menu layoutMenu = new Menu(stMenu.getSWTMenu());
		mi.setMenu(layoutMenu);

		mi = new MenuItem(layoutMenu, SWT.CASCADE | SWT.RADIO);
		mi.setText("Standard");
		mi.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				mailPane.setMailPaneOrientation(SWT.VERTICAL);
			}
		});
		mi.setSelection(settings.mailPaneOrientation == SWT.VERTICAL);

		mi = new MenuItem(layoutMenu, SWT.CASCADE | SWT.RADIO);
		mi.setText("Three-column");
		mi.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				mailPane.setMailPaneOrientation(SWT.HORIZONTAL);
			}
		});
		mi.setSelection(settings.mailPaneOrientation == SWT.HORIZONTAL);

		new MenuItem(viewMenu,  SWT.SEPARATOR, 2);

		menuBar = stMenu.getSWTMenu();

		
		this.pack();
	
	}

	@Override
	public Menu getMenuBar() {
		return menuBar;
	}

	public void newArticles(Collection<ArticleInfo> info) {
		Iterator<ArticleInfo> iter = info.iterator();
		if (iter.hasNext()) calPane.feedUpdated(iter.next().getFeed(), true);
	}

	public void newChannel(Channel channel) {}

	public void newFeed(Feed feed) {}

	public void newVirtual(Virtual virtual) {}
	
	public void terminate() {
		super.terminate();
		downloadManager.removeDownloadObserver(statusBar);
		search.terminate();
		int[] date = calPane.getSelectedDay();
		controller.saveSettings(this.getClass().getName(), 
				new CalendarViewSettings(sashForm.getSize().y, sashForm.getSize().x,
						sashForm.getWeights(),mailPane.getOrientation(),mailPane.getMailPaneWeights(),
						getShell().getLocation(),mailPane.getSortCriterion(),mailPane.getSortDirection(),
						date[0],date[1],date[2],DNDArticle.fromArticleInfo(mailPane.getSelectedArticle()),
						mailPane.getColumnWidthData(),mailPane.getAppearance()));
	}
}
