package view;

import java.util.Collection;
import java.util.Iterator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.LocationEvent;
import org.eclipse.swt.browser.LocationListener;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.ExpandBar;
import org.eclipse.swt.widgets.ExpandItem;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;

import search.SearchBox;
import search.SearchObserver;
import util.IconFactory;
import util.IconFactory.ICON;
import controller.Controller;
import data.ArticleInfo;
import data.Channel;
import data.Container;
import data.Feed;
import data.Virtual;
import data.VirtualFactory;

/**
 * The minimalistic view.
 * @author oleg
 * @author Kuat
 */
public class MinimalisticView extends View implements SearchObserver{
	private ExpandBar expandBar;
	private Virtual standard;
	private Virtual current;
	private final String titleFont;
	private final String articleFont;
	public Menu getMenuBar() {
		return null;
	}
	private final LocationListener locationListener = new LocationListener() {
		public void changed(LocationEvent event) {
			
		}
		public void changing(LocationEvent event) {
			if (event.location.startsWith("http://") || event.location.startsWith("https://")) {
				event.doit = false;
				Program.launch(event.location);
			}
		}
	};
	public MinimalisticView(Shell parent, Controller controller) {
		super(parent, SWT.BORDER, controller);				
		GridLayout thisLayout = new GridLayout(1,false);
		thisLayout.horizontalSpacing = 0;
		thisLayout.verticalSpacing = 0;
		thisLayout.marginHeight = 0;
		thisLayout.marginWidth = 0;
		setLayout(thisLayout);
		Composite bar = new Composite(this, SWT.None);
		bar.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		GridLayout barLayout = new GridLayout(2, false);
		barLayout.marginHeight = 0;
		barLayout.verticalSpacing = 0;
		bar.setLayout(barLayout);
		controller.getActionManager().createStandardToolbar(bar).setLayoutData(new GridData(GridData.FILL_HORIZONTAL));;		
		new SearchBox(controller, bar, this, false, false, false);		
		standard = VirtualFactory.empty("Most recent");
		current = standard;
		FontData[] fd = getFont().getFontData();
		if (fd.length > 0) {
			articleFont = new StringBuffer().append(fd[0].getHeight()).append("pt ").append(fd[0].getName()).toString();
			titleFont = new StringBuffer().append(fd[0].getHeight() + 2).append("pt ").append(fd[0].getName()).toString();
		} else {
			articleFont = "8pt Sans";
			titleFont = "10pt bold Sans";
		}
		populate();	
	}
	 
	
	private void populate() {
		if (expandBar != null) {
			for(ExpandItem item : expandBar.getItems()) {
				item.getControl().dispose();
				item.dispose();
			}
		} else { 		
			expandBar = new ExpandBar(this, SWT.BORDER | SWT.V_SCROLL);
			expandBar.setLayoutData(new GridData(GridData.FILL_BOTH));
		}
		
		Iterator<ArticleInfo> articles = dataManager.getArticles(VirtualFactory.limitVirtual(VirtualFactory.orderByDate(current), 10));
		
		if (articles == null) return;
		
		while(articles.hasNext()) {
			ArticleInfo a = articles.next();
			ExpandItem item = new ExpandItem(expandBar,SWT.NONE);
			item.setText(a.getTitle());
			StringBuffer sb = new StringBuffer();
			sb.append("<html><body style=\"overflow: auto; font: ").append(articleFont);
			sb.append("\"><p><a style=\"font: ").append(titleFont).append("\" href=\"");
			sb.append(a.getURL()).append("\">").append(a.getTitle()).append("</a></p>");
			if (a.getFeed().getIcon() != null)
				item.setImage(new Image(expandBar.getDisplay(),a.getFeed().getIcon()));
			else item.setImage(IconFactory.getIcon(getDisplay(),ICON.FAVICON));
			sb.append(dataManager.getContent(a).getSummary());
			sb.append("</body></html>");
			Browser wb = new Browser(expandBar,SWT.BORDER);
			wb.addLocationListener(locationListener);
			wb.setText(sb.toString());
			item.setHeight(230);
			item.setControl(wb);
		}
	}

	public void newChannel(Channel channel) {
		// ignore
	}

	public void newFeed(Feed feed) {
		// ignore
	}

	public void newVirtual(Virtual virtual) {
		// ignore
	}

	public void newArticles(final Collection<ArticleInfo> info) {
		this.getDisplay().asyncExec(new Runnable() {
			public void run() {
				populate();	
			}
		});
	}

	public void cancelSearch() {
		current = standard;
		populate();			
	}

	public Container currentView() {
		return standard;
	}

	public void displayResults(Virtual search) {
		if ((search != null) && (!search.isFullText())){
			current = search;			
			populate();
		}
	}	
}
