package view;

import java.util.Collection;
import java.util.Iterator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;

import search.SearchBox;
import settings.NewspaperPaneAppearanceDialog;
import settings.NewspaperViewSettings;
import util.IconFactory;
import util.IconFactory.ICON;
import view.components.FolderPane;
import view.components.NewspaperPane;
import view.components.StatusBar;
import controller.Controller;
import controller.StandardMenu;
import data.ArticleInfo;
import data.Channel;
import data.Feed;
import data.Virtual;

/**
 * Newspaper view. Not ready yet.
 * @author oleg
 */
public class NewspaperView extends View {
	
	private final Menu menuBar;
	private final SearchBox search;
	private final SashForm sashForm;
	private final StatusBar statusBar;
	private final FolderPane folderPane;
	private final NewspaperPane newsPane;
	
	public Menu getMenuBar() {
		return menuBar;
	}

	@SuppressWarnings("unchecked")
	public NewspaperView(final Shell shell, Controller ctrl) {
		super(shell, ctrl);
		NewspaperViewSettings settings = (NewspaperViewSettings) controller.getSettings(this.getClass().getName());
		if (settings == null) settings = NewspaperViewSettings.defaultSettings();


		GridLayout newsViewLayout = new GridLayout(2,false);
		newsViewLayout.horizontalSpacing = 0;
		newsViewLayout.marginWidth = 0;
		newsViewLayout.marginHeight = 0;
		newsViewLayout.verticalSpacing = 0;
		this.setLayout(newsViewLayout);



		Composite bar = new Composite(this, SWT.None);
		bar.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		GridLayout barLayout = new GridLayout(2, false);
		barLayout.verticalSpacing = 0;
		barLayout.marginHeight = 0;
		bar.setLayout(barLayout);
		controller.getActionManager().createStandardToolbar(bar).setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		sashForm = new SashForm(this,SWT.HORIZONTAL);

		folderPane = new FolderPane(sashForm,controller);

		newsPane = new NewspaperPane(sashForm, controller, settings.appearance);
		
		search = new SearchBox(controller, bar, newsPane, false, true, false);
		
		sashForm.setWeights(settings.sashWeights);

		for(Channel c: dataManager.getChannels()) {
			if (!c.getName().equals("")) folderPane.addChannel(c);
		}

		for(Feed f: dataManager.getFeeds()) {
			folderPane.addFeed(f);
		}

		for(Virtual v: dataManager.getVirtuals())
			folderPane.addVirtual(v);

		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.horizontalSpan = 2;
		gd.heightHint = settings.mainHeight;
		gd.widthHint = settings.mainWidth;
		sashForm.setLayoutData(gd);

		statusBar = new StatusBar(this, controller, SWT.BORDER);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		statusBar.setLayoutData(gd);

		StandardMenu stMenu = controller.getActionManager().createStandardMenuBar(); 
		Menu viewMenu = stMenu.getSubMenu("View");
		MenuItem mi = new MenuItem(viewMenu,SWT.CASCADE,0);
		mi.setText("Fonts and colors...");
		mi.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				new NewspaperPaneAppearanceDialog(shell,newsPane);
			}
		});
		mi.setImage(IconFactory.getIcon(getDisplay(), ICON.FONTS_AND_COLORS));
		mi = new MenuItem(viewMenu,SWT.SEPARATOR,1);
		menuBar = stMenu.getSWTMenu();
		folderPane.addContainerSelectionObserver(newsPane);
		downloadManager.addDownloadObserver(statusBar);		

		folderPane.setSelection(settings.folderPaneSelection[0], settings.folderPaneSelection[1]);

		this.pack();
	}

	@Override
	public void terminate() {
		controller.saveSettings(this.getClass().getName(), new NewspaperViewSettings(sashForm.getSize().x, sashForm.getSize().y, sashForm.getWeights(), folderPane.getSelection(),newsPane.getAppearance()));
		super.terminate();
		search.terminate();
		downloadManager.removeDownloadObserver(statusBar);
	}

	public void newChannel(final Channel channel) {
		this.getDisplay().asyncExec(new Runnable() {
			public void run() {
				folderPane.addChannel(channel);
			}
		});
	}

	public void newFeed(final Feed feed) {
		this.getDisplay().asyncExec(new Runnable() {
			public void run() {
				folderPane.addFeed(feed);
			}
		});
	}

	public void newVirtual(final Virtual virtual) {
		this.getDisplay().asyncExec(new Runnable() {
			public void run() {
				folderPane.addVirtual(virtual);
			}
		});
	}
	
	public void newArticles(Collection<ArticleInfo> info) {
		Iterator<ArticleInfo> iter = info.iterator();
		if (iter.hasNext()) {
			Feed feed = iter.next().getFeed();
			folderPane.feedUpdated(feed, true); 
			newsPane.feedUpdated(feed);
		}
	}


	

}
