package view;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

import controller.Controller;
import data.DataManager;
import download.DownloadManager;

/**
 * The abstract class of a view component. Every view has to extend this class
 * to be recognized as a valid view component.
 * @author Kuat
 *
 */
public abstract class View extends Composite implements RSSObserver {
	
	protected final Controller controller;
	protected final DataManager dataManager;
	protected final DownloadManager downloadManager;
	
	protected View (Shell shell, int style, Controller controller) {
		super(shell, style);
		this.controller = controller;
		this.dataManager = controller.getDataManager();
		this.downloadManager = controller.getDownloadManager();
	}
	
	public View (Shell shell, Controller controller) {
		this(shell, SWT.NONE, controller);
	}
	
	public void reportError(final String message, final Shell shell) {
		this.getDisplay().asyncExec(new Runnable() {
			public void run() {
				MessageBox mb = new MessageBox(shell, SWT.ICON_ERROR | SWT.OK | SWT.PRIMARY_MODAL);
				mb.setText("Error");
				if (message != null)
					mb.setMessage(message);
				mb.open();
			}
		});
	}
	
	/**
	 * The controller guarantees that this method will be called before the
	 * view disposal.
	 */
	public void terminate() {
		dataManager.removeViewListener(this);
	}
	
	/**
	 * The menu bar to set when the view is activated.
	 * @return a menu or null if no menu bar should be displayed
	 */
	public abstract Menu getMenuBar();
}
