package view.components;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DateTime;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;


import data.Feed;
import data.Virtual;
import data.VirtualFactory;
/**
 * Calendar pane. 
 * @author oleg
 */
public class CalendarPane extends Composite implements ContainerSelector {
	private final Calendar calUtil = new GregorianCalendar();
	private Vector<ContainerSelectionObserver> selectionListeners = new Vector<ContainerSelectionObserver>(1);
	private final DateTime calendar;
	
	public void setSelectedDay(int year, int month, int day) {
		calendar.setYear(year);
		calendar.setMonth(month);
		calendar.setDay(day);
		notifyObservers(false);
	}
	
	public int[] getSelectedDay() {
		return new int[] {calendar.getYear(),calendar.getMonth(),calendar.getDay()};
	}
	
	public CalendarPane(Composite parent) {
		super(parent, SWT.NONE);
		setLayout(new FillLayout());
		calendar = new DateTime(this, SWT.CALENDAR);
		calendar.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				notifyObservers(false);
			}
		});
	}
	
	public void addContainerSelectionObserver(ContainerSelectionObserver l) {
		synchronized (this) {
			selectionListeners.add(l);
		}
	}
	
	@SuppressWarnings("unchecked")
	private void notifyObservers(boolean keepSelection) {
		int year = calendar.getYear();
		int month = calendar.getMonth();
		int day = calendar.getDay();
		calUtil.set(year, month, day, 0, 0);
		long start = calUtil.getTimeInMillis();
		calUtil.add(Calendar.DAY_OF_MONTH, 1);
		long end = calUtil.getTimeInMillis() - 1;
		Virtual virtual = VirtualFactory.dateRange(start, end);

		Vector<ContainerSelectionObserver> l;
		synchronized (this) {
			l = (Vector<ContainerSelectionObserver>) selectionListeners.clone();
		}
		for (ContainerSelectionObserver sl: l) {
			sl.virtualSelected(virtual, keepSelection);
		}
	}
	
	public void feedUpdated(Feed feed, boolean newArticles) {
		if (newArticles) getDisplay().asyncExec(new Runnable() {
			public void run() {
				notifyObservers(true);
			}
		});
	}

}
