package view.components;

import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.LocationEvent;
import org.eclipse.swt.browser.LocationListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Composite;

import search.SearchObserver;
import settings.NewspaperPaneAppearance;
import util.FileUtil;
import controller.Controller;
import data.ArticleInfo;
import data.Channel;
import data.Container;
import data.Feed;
import data.Virtual;
import data.VirtualFactory;

/**
 * A component which presents a collection of RSS articles as a 3-column newspaper page.
 * @author oleg
 */
public class NewspaperPane extends Composite implements ContainerSelectionObserver, SearchObserver {
	private final Browser browser;
	private final Controller controller;
	private String currentHTMLBody = null;
	private final static String PAGE_JAVASCRIPT = "\nfunction showDialog() {\n" +
	"elt=document.getElementById(\"refreshDialog\");\n" +
	"elt.style.left = document.body.scrollLeft + 5;\n" +
	"elt.style.top = document.body.scrollTop + 5;\n" +
	"elt.style.visibility = \"visible\";\n" +
	"}\n" +
	"function closeDialog() {\n" +
	"document.getElementById(\"refreshDialog\").style.visibility=\"hidden\";\n" +
	"}\n";

	private String refreshImageURL = "";
	private Container currentContainer = null;
	private NewspaperPaneAppearance appearance;
	
	public void setAppearance(NewspaperPaneAppearance appearance) {
		this.appearance = appearance;
		if (currentHTMLBody != null) {
			browser.setText(generateHTMLPage(currentHTMLBody));
		}
	}
	public NewspaperPaneAppearance getAppearance() {
		return appearance;
	}

	private final String generate3ColumnHTMLBody(String title, Collection<ArticleInfo> column1, Collection<ArticleInfo> column2, Collection<ArticleInfo> column3) {
		StringBuffer xhtml = new StringBuffer("<body>"); 		
		xhtml.append("<div class=\"dialog\" id=\"refreshDialog\" style=\"position:absolute; width:200px; height:60px; padding: 5px;" +
				"background-color:white; z-index:999; color:black; font: 8pt Tahoma; border:1px solid black; visibility:hidden\"" +
		"><p>This feed has been updated</p><p><a href=\"ignore://\">Ignore</a> <a href=\"refresh://\">Refresh</a></p></div>");
		xhtml.append("<div style=\"position:absolute; top:0px; right:0px\"><a href=\"refresh://\"><img style=\"border: none\" id=\"refreshImage\" src=\"").append(refreshImageURL).append("\" /></a></div>\n");
		xhtml.append(generateHeaderDiv(title));
		xhtml.append(generateColumn("left: 2%", column1));
		xhtml.append(generateColumn("left: 34%", column2));
		xhtml.append(generateColumn("left: 66%", column3));
		xhtml.append("</body>");
		return xhtml.toString();
	}

	private final String generateHTMLPage(String body) {
		StringBuffer xhtml = new StringBuffer("<html><head><meta/><title></title>");
		xhtml.append("<style type=\"text/css\">");
		xhtml.append(appearance.getCSS());
		xhtml.append("</style>\n<script>");
		xhtml.append(PAGE_JAVASCRIPT).append("\n</script>").append("</head>");
		xhtml.append(body).append("</html>");
		return xhtml.toString();
	}

	private final StringBuffer generateColumn(String alignStyle, Collection<ArticleInfo> column) {
		StringBuffer div = new StringBuffer("<div class=\"rss-column\" style=\"");
		div.append(alignStyle).append("\">");
		for (ArticleInfo article: column) div.append(generateArticleDiv(article));
		div.append("</div>");
		return div;
	}

	private final StringBuffer generateArticleDiv(ArticleInfo article) {
		StringBuffer div = new StringBuffer("<a class=\"rss-header\" href=\"");
		div.append(article.getURL()).append("\">").append(article.getTitle());
		div.append("</a>").append("<div class=\"rss-author\">\n");
		if (!article.getAuthors().equals("")) div.append("By ").append(article.getAuthors());
		div.append("</div><div class=\"rss-article\">\n");
		div.append(controller.getDataManager().getContent(article).getSummary()).append("</div>\n");
		return div;
	}
	private final StringBuffer generateHeaderDiv(String title) {
		return new StringBuffer("<div class=\"rss-main\">").append(title).append("</div>\n");
	}
	private final void show(String title, Iterator<ArticleInfo> iter, int max) {		
		List<ArticleInfo> column1 = new LinkedList<ArticleInfo>();
		List<ArticleInfo> column2 = new LinkedList<ArticleInfo>();
		List<ArticleInfo> column3 = new LinkedList<ArticleInfo>();
		int count = 0;

		if (iter == null)
			return;

		while (iter.hasNext() && count < max) {
			column1.add(iter.next());
			count ++;
			if (iter.hasNext() && count < max) {
				column2.add(iter.next());
				count ++;
				if (iter.hasNext() && count < max) {
					column3.add(iter.next());
					count ++;
				}
			}
		}
		currentHTMLBody = generate3ColumnHTMLBody(title, column1, column2, column3); 
		browser.setText(generateHTMLPage(currentHTMLBody));
	}

	
	public NewspaperPane(Composite parent, Controller controller, NewspaperPaneAppearance appearance) {
		super(parent,SWT.NONE);
		setLayout(new FillLayout());
		this.appearance = appearance;
		this.controller = controller;
		// TODO: change when packaging
//		String separator = System.getProperty("file.separator");
//		String htmlLocation = "";
//		try  {
//			htmlLocation = new StringBuffer("file://").append(new File(".").getCanonicalPath()).append(separator).append("html").append(separator).toString();
//		} catch (IOException e) {
//			e.printStackTrace();
//		}
//		refreshImageURL = htmlLocation  + "refresh.png";
		//////////////////////
		browser = new Browser(this,SWT.NONE);
		
		try {
			refreshImageURL = "file://" + FileUtil.extractResource(getClass(), "/html/refresh.png", "refresh", ".png");
		} catch (IOException e) {
			controller.reportError("An error occurred while extracting a temporary file:\n" + e.getMessage());
		}
		
		browser.addLocationListener(new LocationListener() {
			public void changed(LocationEvent event) {

			}
			public void changing(LocationEvent event) {
				// Kuat: equals does not work on WIndows IE
				if (event.location.startsWith("ignore://")) {
					event.doit = false;
					browser.execute("closeDialog()");
				} else if (event.location.startsWith("refresh://")) {
					event.doit = false;
//					browser.execute("closeDialog()");
					refreshView();
				} else if (event.location.startsWith("http://") || event.location.startsWith("https://")) {
					Program.launch(event.location);
					event.doit = false;
				}
			}
		});

	}
	
	private void refreshView(){
		if (currentContainer != null) {
			if (currentContainer instanceof Feed) {
				feedSelected((Feed) currentContainer);
			} else if (currentContainer instanceof Virtual) {
				virtualSelected((Virtual) currentContainer, false);
			} else if (currentContainer instanceof Channel) {
				channelSelected((Channel) currentContainer);
			}		
		}
	}
	
	public void cancelSearch() {
		refreshView();
	}

	public Container currentView() {
		return currentContainer;
	}

	public void displayResults(Virtual virtual) {
		show(virtual.getName(),controller.getDataManager().getArticles(virtual),30);		
	}

	public void channelSelected(Channel channel) {
		currentContainer = channel;
		Virtual virtual = VirtualFactory.orderByDate(VirtualFactory.createVirtualFromChannel(channel));
		show(virtual.getName(),controller.getDataManager().getArticles(virtual),30);
	}
	
	public void feedSelected(Feed feed) {
		currentContainer = feed;
		Virtual virtual = VirtualFactory.orderByDate(VirtualFactory.createVirtualFromFeed(feed, false));
		show(feed.getName(),controller.getDataManager().getArticles(virtual),30);
	}

	public void outboxSelected(Feed feed) {
		feedSelected(feed);
	}

	public void virtualSelected(Virtual virtual, boolean keepSelection) {
		currentContainer = virtual;
		if (!virtual.isFullText()) virtual = VirtualFactory.orderByDate(virtual);
		if (virtual == null) return;
		show(virtual.getName(),controller.getDataManager().getArticles(virtual),30);		
	}
	
	/**
	 * Brings up a dialog asking whether to refresh the page if the given feed is currently displayed.
	 * @param feed the feed that has been updated
	 */
	public void feedUpdated(Feed feed) {
		if (feed.equals(currentContainer)) {
			getDisplay().asyncExec(new Runnable() {
				public void run() {
					browser.execute("showDialog();");
				}
			});
		}
	}
	
}
