package view.components;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.ProgressBar;

import util.IconFactory;

import controller.Controller;
import data.Feed;
import download.DownloadObserver;

/**
 * A status bar capable of displaying the download manager status.
 * @author oleg
 */
public class StatusBar extends Composite implements DownloadObserver {
	private final Label connectionIcon;
	private final Label status;
	private final ProgressBar progressBar;
	private final Image offline;
	private final Image online;
	private final MenuItem onlineItem;
	private final MenuItem offlineItem;
	/**
	 * Creates a new status bar
	 * @param parent parent pane
	 * @param style SWT style
	 */
	public StatusBar(final Composite parent, final Controller controller, int style) {
		super(parent, style);
		
		online = new Image(this.getDisplay(),IconFactory.getIcon(IconFactory.ICON.ONLINE));
		offline = new Image(this.getDisplay(),IconFactory.getIcon(IconFactory.ICON.OFFLINE));
		setLayout(new FormLayout());
		progressBar = new ProgressBar(this, SWT.HORIZONTAL | SWT.INDETERMINATE | SWT.BORDER);
		FormData fd = new FormData();
		fd.right = new FormAttachment(100,0);
		fd.left = new FormAttachment(75,0);
		progressBar.setLayoutData(fd);
		progressBar.setVisible(false);		
		connectionIcon = new Label(this,SWT.NONE | SWT.BORDER);

		fd = new FormData();
		fd.right = new FormAttachment(progressBar,0);

		connectionIcon.setLayoutData(fd);
		connectionIcon.setImage(controller.isOnline()? online : offline);
		
		status = new Label(this, SWT.NONE);
		fd = new FormData();
		fd.right = new FormAttachment(connectionIcon,0);
		fd.left = new FormAttachment(0,0);
		
		status.setLayoutData(fd);
		

		final Menu popupMenu = new Menu(connectionIcon);
		connectionIcon.setMenu(popupMenu);
		onlineItem = new MenuItem(popupMenu, SWT.RADIO);
		onlineItem.setText("Online");
		onlineItem.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				connectionIcon.setImage(online);
				controller.setOnline(true);
			}
		});
		offlineItem = new MenuItem(popupMenu, SWT.RADIO);
		offlineItem.setText("Offline");
		offlineItem.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				connectionIcon.setImage(offline);
				controller.setOnline(false);
			}
		});
		if (controller.isOnline()) onlineItem.setSelection(true);
		else offlineItem.setSelection(true);
	}
	/**
	 * Reflects download finish
	 */
	public void downloadFinished() {
		if (!isDisposed()) {
			this.getDisplay().asyncExec(new Runnable() {
				public void run() {
					if (!isDisposed()) {
						status.setText("");
						progressBar.setVisible(false);
					}
				}
			}); 
		}
	}
	/**
	 * Reflects download start
	 */
	public void downloadStarted(final Feed feed) {
		if (!isDisposed()) {
			this.getDisplay().asyncExec(new Runnable() {
				public void run() {					
					if (!isDisposed()) {
						if (feed != null)
							status.setText("Downloading " + feed.getURL());
						else
							status.setText("Checking feed");
						progressBar.setVisible(true);
					}
				}
			});
		}
	}
}
