package view.dialogs;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

import util.IconFactory;
import util.IconFactory.ICON;
import controller.ActionManager;
import controller.Controller;
import controller.GlobalPreferences;

public class AboutBox {
	private final Shell shell;
	
	private final Listener urlListener = new Listener() {
		public void handleEvent(Event event) {
			Program.launch(event.text);
		}
	};
	
	public AboutBox(Shell parent, final Controller controller) {
		shell = new Shell(parent, SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM);
		shell.setText("About " + GlobalPreferences.APPLICATION_NAME);
		shell.setImage(IconFactory.getIcon(parent.getDisplay(), IconFactory.ICON.STAR));
		shell.setLayout(new GridLayout(3,true));
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.horizontalAlignment = SWT.CENTER;
		gd.horizontalSpan = 3;
		Label logo = new Label(shell, SWT.NONE | SWT.BORDER);
		logo.setImage(IconFactory.getIcon(shell.getDisplay(), ICON.LOGO));
		logo.setLayoutData(gd);
		Label label = new Label(shell, SWT.NONE);
		label.setText(GlobalPreferences.APPLICATION_NAME + " has been developed by");
		
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalAlignment = SWT.CENTER;
		gd.horizontalSpan = 3;

		label.setLayoutData(gd);

		Link a = new Link(shell, SWT.NONE);
		a.setText("<A HREF=\"mailto:kuat@mit.edu\">Kuat Yessenov</A>");
		a.addListener(SWT.Selection, urlListener);
		a.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_CENTER));
		
		a = new Link(shell, SWT.NONE);
		a.setText("<A HREF=\"mailto:golberg@mit.edu\">Oleg Golberg</A>");
		a.addListener(SWT.Selection, urlListener);
		a.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_CENTER));
		
		a = new Link(shell, SWT.NONE);
		a.setText("<A HREF=\"mailto:wajahat@mit.edu\">Wajahat Khan</A>");
		a.addListener(SWT.Selection, urlListener);
		a.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_CENTER));

		a = new Link(shell, SWT.NONE);
		a.setText("Tango icons are used under the terms of\n" +
				"<A HREF=\"http://creativecommons.org/licenses/by-sa/2.5/\">Creative Commons Attribution-ShareAlike</A>");
		a.addListener(SWT.Selection, urlListener);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalAlignment = SWT.CENTER;
		gd.horizontalSpan = 3;

		a.setLayoutData(gd);

		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
				
		Composite buttons = new Composite(shell, SWT.NONE);
		buttons.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		buttons.setLayoutData(gd);
		
		buttons.setLayout(new GridLayout(2, false));
		Button button = new Button(buttons, SWT.NONE);
		button.setText("Visit the web page");

		button.addListener(SWT.Selection, controller.getActionManager().getAction(ActionManager.Actions.VISIT_WEBSITE));
		button = new Button(buttons, SWT.NONE);
		button.setText("Close");

		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				shell.close();
			}
		});

		
		
		
		shell.pack();
		shell.open();
		while (!shell.isDisposed()) {
			if (!shell.getDisplay().readAndDispatch()) 
				shell.getDisplay().sleep();
		}
	}
}
