package view.dialogs;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;

import controller.Controller;
import data.DataException;
import data.Feed;

public class ChangeIntervalDialog {
	/**
	 * Creates a new dialog with the given parent shell and controller to change the update interval for a given feed.
	 * @param parent parent shell
	 * @param controller controller
	 * @param feed feed
	 */
	public ChangeIntervalDialog(final Shell parent, final Controller controller, final Feed feed) {
		final Shell shell = new Shell(parent,SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText("Set update interval");
		shell.setLayout(new GridLayout(1,false));
		Composite input = new Composite(shell, SWT.NONE);
		input.setLayoutData(new GridData(GridData.FILL_BOTH));

		input.setLayout(new GridLayout(3,false));

		Label label = new Label(input,SWT.NONE);
		label.setText("Update every");

		final Spinner updateInterval = new Spinner(input,SWT.BORDER);
		updateInterval.setMinimum(1);
		updateInterval.setMaximum(60);
		updateInterval.setSelection((int) feed.getUpdateInterval()/60000);

		label = new Label(input,SWT.NONE);
		label.setText("minutes");
		
		Composite buttons = new Composite(shell, SWT.NONE);
		buttons.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		buttons.setLayout(new GridLayout(2, false));
		Button button = new Button(buttons, SWT.NONE);
		button.setText("Set");

		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				try {
					controller.getDataManager().setUpdateInterval(feed, 60000L*updateInterval.getSelection());
					shell.close();
				} catch (DataException e) {
					controller.reportError("Changing the update interval caused an error:\n" + e.getMessage(), shell);
				}
			}
		});

		button = new Button(buttons, SWT.NONE);
		button.setText("Cancel");

		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				shell.close();
			}
		});

		shell.pack();
		shell.open();
		while (!shell.isDisposed()) {
			if (!shell.getDisplay().readAndDispatch()) {
				shell.getDisplay().sleep();
			}
		}
	}
}
