package view.dialogs;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.htmlparser.NodeFilter;
import org.htmlparser.Parser;
import org.htmlparser.filters.NodeClassFilter;
import org.htmlparser.tags.LinkTag;
import org.htmlparser.util.NodeList;
import org.htmlparser.util.ParserException;

import controller.Controller;
import data.DataException;

/**
 * A dialog for importing craigslist listings RSS feeds.
 * @author Kuat
 *
 */
public class CraigslistImportDialog {
	private final Shell shell;
	private final Controller controller;
	private static Map<String, String> links;
	private final String channelName = "Map feeds";
	private final Text folder;
	final Combo combo;
	
	/**
	 * Creates a new dialog with the given parent shell and controller.
	 * @param parent parent shell
	 * @param controller controller
	 */
	public CraigslistImportDialog(final Shell parent, final Controller controller) {
		this.controller = controller;
		shell = new Shell(parent,SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText("Craigslist Import");
		shell.setLayout(new GridLayout(1,false));
		
		Composite input = new Composite(shell, SWT.NONE);
		input.setLayoutData(new GridData(GridData.FILL_BOTH));
		input.setLayout(new GridLayout(2,false));
		
		new Label(input, SWT.None).setText("Select a city");
		combo = new Combo(input, SWT.BORDER | SWT.READ_ONLY);
		combo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));		
		createLinks();

		List<String> list = new ArrayList<String>();
		for (String link:links.keySet())
			list.add(link);
		
		Collections.sort(list);

		for (String link:list)  
			combo.add(link);
		
		combo.select(0);
		
		Label label = new Label(input,SWT.NONE);
		label.setText("New folder");

		folder = new Text(input, SWT.BORDER | SWT.SINGLE);
		folder.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		folder.setText(channelName);

		Composite buttons = new Composite(shell, SWT.NONE);
		buttons.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		buttons.setLayout(new GridLayout(2, false));
		Button button = new Button(buttons, SWT.NONE);
		button.setText("Subscribe");

		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				subscribe();
			}
		});

		button = new Button(buttons, SWT.NONE);
		button.setText("Cancel");

		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				shell.close();
			}
		});

		shell.pack();
		shell.open();
		while (!shell.isDisposed()) {
			if (!shell.getDisplay().readAndDispatch()) {
				shell.getDisplay().sleep();
			}
		}

	}

	private void subscribe() {
		final String selection = combo.getText();
		final String link = links.get(selection);
		final String folderName = folder.getText();

		new Thread() {
			public void run() {
				try {
					controller.getDataManager().addChannel(folderName);
				} catch (DataException e) {
					// ignore
				}

				controller.getDownloadManager().newSubscription(link, selection, controller.getDataManager().getChannel(folderName), 60000L);					
			}
		}.start();
		shell.close();
	}

	/**
	 * Extract all links from a web page on craigslist.com.
	 *
	 */
	private void createLinks() {
		if (links != null) return;
		
		links = new HashMap<String, String>();
        Parser parser;
        NodeFilter filter;
        NodeList list;
        
        filter = new NodeClassFilter (LinkTag.class);
        
        try
        {
            parser = new Parser ("http://geo.craigslist.org/iso/us");
            list = parser.extractAllNodesThatMatch (filter);
            for (int i = 0; i < list.size (); i++) {
            	LinkTag tag = (LinkTag) list.elementAt (i);
                String link = tag.extractLink();
                if (link.endsWith("craigslist.org/") && !link.equals("http://www.craigslist.org/"))
                	links.put(tag.getLinkText(), link + "hhh/index.rss");
            }
        }
        catch (ParserException e)
        {
            e.printStackTrace ();
        }
	}
}
