package view.dialogs;


import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

import controller.Controller;
import data.Feed;

/**
 * A dialog for converting RSS feeds to their GeoRSS analogs.
 * @author Kuat
 *
 */
public class GeoRSSConverter {
	private final Shell shell;
	private final Controller controller;
	private final static String key = "http://ws.geonames.org/rssToGeoRSS?feedUrl=";
	final Combo combo;
	final List<Feed> feeds;

	/**
	 * Creates a new dialog with the given parent shell and controller.
	 * @param parent parent shell
	 * @param controller controller
	 */
	public GeoRSSConverter(final Shell parent, final Controller controller) {
		this.controller = controller;
		shell = new Shell(parent,SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText("GeoRSS Converter");
		shell.setLayout(new GridLayout(1,false));
		
		Composite input = new Composite(shell, SWT.NONE);
		input.setLayoutData(new GridData(GridData.FILL_BOTH));
		input.setLayout(new GridLayout(2,false));

		new Label(input, SWT.None).setText("Select a feed");
		combo = new Combo(input, SWT.BORDER | SWT.READ_ONLY);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.widthHint = 200;
		combo.setLayoutData(gd);		

		feeds = new ArrayList<Feed>();
		for (Feed feed : controller.getDataManager().getFeeds()) {
			if (feed.isLocal() || feed.getURL().startsWith(key))
				continue;
			combo.add(feed.getName() + " (" + feed.getURL() + ")");
			feeds.add(feed);
		}

		combo.select(0);	
						
		Composite buttons = new Composite(shell, SWT.NONE);
		buttons.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		buttons.setLayout(new GridLayout(2, false));
		Button button = new Button(buttons, SWT.NONE);
		button.setText("Subscribe");

		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				subscribe();
			}
		});

		button = new Button(buttons, SWT.NONE);
		button.setText("Cancel");

		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				shell.close();
			}
		});

		shell.pack();
		shell.open();
		while (!shell.isDisposed()) {
			if (!shell.getDisplay().readAndDispatch()) {
				shell.getDisplay().sleep();
			}
		}

	}

	private void subscribe() {
		final Feed feed = feeds.get(combo.getSelectionIndex());

		new Thread() {
			public void run() {
				controller.getDownloadManager().newSubscription(key + feed.getURL(), feed.getName() + " (GeoRSS)", feed.getChannel(), feed.getUpdateInterval());
			}
		}.start();
		shell.close();
	}
}
