package view.dialogs;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

import util.IconFactory;
import util.Pair;
import util.IconFactory.ICON;
import webapi.GoogleMapsWidget;
import webapi.NetGeo;
import webapi.OnLoadListener;
import webapi.WebConnectionException;
import webapi.GoogleMapsWidget.ZoomControl;
import controller.Controller;
import data.DataManager;
import data.Feed;


/**
 * A dialog which shows the locations of feeds on a map (using NetGeo and Google Maps).
 * @author oleg
 */
public class GoogleMapShowFeeds implements OnLoadListener {
	private final GoogleMapsWidget mapPane;
	private final Controller controller;
	private final DateFormat dateFormatter = new SimpleDateFormat("MM/dd/yy HH:mm:ss");
	private String getHTMLInfo(Feed feed) {
		DataManager dm = controller.getDataManager();
		int total = dm.numberArticles(feed);
		int unread = dm.unreadArticles(feed);
		StringBuffer html = new StringBuffer("<div style=\"font: 8pt Tahoma\">");
		html.append("<p style=\"font: 10pt Tahoma; text-align: center\">").append(feed.getName()).append("</p>");
		html.append(unread).append(" unread/").append(total).append(" total <br />");
		html.append("Last update: ").append(dateFormatter.format(new Date(feed.getSyncTime()))); 
		html.append("</div>");
		return html.toString();
	}
	/**
	 * Displays all feeds for which a NetGeo location is available.
	 */
	public void onLoad() {
		new Thread() {
			public void run() {
				for (Feed feed: controller.getDataManager().getFeeds()) {
					if (!feed.isLocal()) {
						Pair<String, String> location = null;
						try {
							location = NetGeo.getLocation(feed.getURL());
						} catch (WebConnectionException e) {
							// should we do anything here?
						} 
						if (location != null) mapPane.addMarker(location.car, location.cdr, getHTMLInfo(feed), true);
					}
				}
			}
		}.start();
	}
	/**
	 * Creates a new dialog with the given parent shell and controller.
	 * @param parent parent shell
	 * @param controller controller
	 */
	public GoogleMapShowFeeds(final Shell parent, final Controller controller) {
		this.controller = controller;
		final Shell shell = new Shell(parent, SWT.APPLICATION_MODAL | SWT.RESIZE | SWT.DIALOG_TRIM);
		shell.setText("Feed locations");
		shell.setImage(IconFactory.getIcon(shell.getDisplay(), ICON.GOOGLE_MAPS));
		shell.setLayout(new FillLayout());
		mapPane = new GoogleMapsWidget(shell, SWT.NONE, ZoomControl.SWT_SCALE, this);
		shell.addListener(SWT.Close, new Listener() {
			public void handleEvent(Event event) {
				controller.saveSettings(GoogleMapShowFeeds.class.getName(), shell.getSize());
			}
		});
		Point size = (Point) controller.getSettings(this.getClass().getName());
		if (size == null) size = new Point(500,400);	
		shell.setSize(size);
		shell.open();

		while (!shell.isDisposed())
			if (!shell.getDisplay().readAndDispatch())
				shell.getDisplay().sleep();

	}
}
