package view.dialogs;

import java.util.Collection;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;

import util.Pair;
import webapi.LiveJournal;
import webapi.WebConnectionException;
import controller.Controller;
import data.Channel;
import data.DataException;

/**
 * A dialog for importing a LiveJournal friendlist as a set of feeds.
 * @author oleg
 */
public class LJImportDialog {
	private final Text username;
	private final Text password;
	private final Text folder;
	private final Spinner syncSpinner;
	private final Shell shell;
	private final Controller controller;
	/**
	 * Creates a new dialog with the given parent shell and controller.
	 * @param parent parent shell
	 * @param controller controller
	 */
	public LJImportDialog(final Shell parent, final Controller controller) {
		this.controller = controller;
		shell = new Shell(parent,SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText("LiveJournal Import");
		shell.setLayout(new GridLayout(1,false));
		Composite input = new Composite(shell, SWT.NONE);
		input.setLayoutData(new GridData(GridData.FILL_BOTH));

		input.setLayout(new GridLayout(3,false));

		Label label = new Label(input,SWT.NONE);
		label.setText("Username");

		username = new Text(input, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		gd.widthHint = 200;
		username.setLayoutData(gd);

		label = new Label(input,SWT.NONE);
		label.setText("Password");

		password = new Text(input, SWT.BORDER | SWT.SINGLE | SWT.PASSWORD);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		password.setLayoutData(gd);

		label = new Label(input,SWT.NONE);
		label.setText("New folder");

		folder = new Text(input, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		folder.setLayoutData(gd);

		label = new Label(input,SWT.NONE);
		label.setText("Update every");

		syncSpinner = new Spinner(input,SWT.BORDER);
		syncSpinner.setMaximum(60);
		syncSpinner.setMinimum(1);
		syncSpinner.setSelection(5);

		label = new Label(input,SWT.NONE);
		label.setText("minutes");
		label.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Composite buttons = new Composite(shell, SWT.NONE);
		buttons.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		buttons.setLayout(new GridLayout(2, false));
		Button button = new Button(buttons, SWT.NONE);
		button.setText("Import");

		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				LJImport();
			}
		});

		button = new Button(buttons, SWT.NONE);
		button.setText("Cancel");

		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				shell.close();
			}
		});

		shell.pack();
		shell.open();
		while (!shell.isDisposed()) {
			if (!shell.getDisplay().readAndDispatch()) {
				shell.getDisplay().sleep();
			}
		}

	}
	// performs the actual import business
	private void LJImport() {
		new Thread() {
			private final String user = username.getText();
			private final String pass = password.getText();
			private final String folderName = folder.getText();
			private long updateInterval = 60000L*syncSpinner.getSelection();
			public void run() {

					Collection<Pair<String, String>> friendsFeeds = null;
					
					try {
						friendsFeeds = LiveJournal.getFriendsFeeds(user, pass);
					} catch (WebConnectionException e) {
						controller.reportError(e.toString());
						return;
					}
					Channel channel = controller.getDataManager().getChannel(folderName);
					if (channel == null) {
						try {
							channel = controller.getDataManager().addChannel(folderName);
						} catch (DataException e) {
							controller.reportError(e.getMessage());
							return;
						}
					}
					for (Pair<String, String> friend: friendsFeeds) {
						controller.getDownloadManager().newSubscription(friend.cdr, friend.car, channel, updateInterval);
					}
			}
		}.start();
		shell.close();
	}
}
