package view.dialogs;

import java.util.Iterator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import parser.RSSParser;
import webapi.LiveJournal;
import webapi.WebConnectionException;
import controller.Controller;
import data.ArticleInfo;

/**
 * A simple dialog for posting an summary of the outbox to LiveJournal.
 * @author oleg
 */
public class LJPostDialog {
	private final Text username;
	private final Text password;
	private final Text title;
	private final Shell shell;
	/**
	 * Creates a new dialog with given parent shell and controller.
	 * @param parent parent shell
	 * @param controller controller
	 */
	public LJPostDialog(final Shell parent, final Controller controller) {
		shell = new Shell(parent,SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText("Post outbox to LiveJournal");
		shell.setLayout(new GridLayout(1,false));
		Composite input = new Composite(shell, SWT.NONE);
		input.setLayoutData(new GridData(GridData.FILL_BOTH));

		input.setLayout(new GridLayout(2,false));

		Label label = new Label(input,SWT.NONE);
		label.setText("Username");

		username = new Text(input, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.widthHint = 200;
		username.setLayoutData(gd);

		label = new Label(input,SWT.NONE);
		label.setText("Password");

		password = new Text(input, SWT.BORDER | SWT.SINGLE | SWT.PASSWORD);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		password.setLayoutData(gd);

		label = new Label(input,SWT.NONE);
		label.setText("Title");

		title = new Text(input, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		title.setLayoutData(gd);

		Composite buttons = new Composite(shell, SWT.NONE);
		buttons.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		buttons.setLayout(new GridLayout(2, false));
		final Button button = new Button(buttons, SWT.NONE);
		button.setText("Post");

		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				button.setEnabled(false);
				final String user = username.getText();
				final String pass = password.getText();
				final String name = title.getText();
				new Thread() {
					public void run() {
						Iterator<ArticleInfo> articles = controller.getDataManager().getArticles(controller.getDataManager().getFeed("rss-reader://outbox"));
						try {
							LiveJournal.postEntry(user, pass, name, RSSParser.getShortHTMLSummary(articles));
							if (!shell.isDisposed()) {
								shell.getDisplay().asyncExec(new Runnable() {
									public void run() {
										if (!shell.isDisposed()) shell.close();
									}
								});
							}
						} catch (WebConnectionException e) {
							controller.reportError(e.getMessage(), shell);
							if (!shell.isDisposed()) {
								shell.getDisplay().asyncExec(new Runnable() {
									public void run() {
										if (!button.isDisposed()) button.setEnabled(true);
									}
								});
							}
						} 
					}
				}.start();
			}
		});

		Button button2 = new Button(buttons, SWT.NONE);
		button2.setText("Cancel");

		button2.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				shell.close();
			}
		});

		shell.pack();
		shell.open();
		while (!shell.isDisposed()) {
			if (!shell.getDisplay().readAndDispatch()) {
				shell.getDisplay().sleep();
			}
		}

	}

}
