package view.dialogs;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import controller.Controller;
import data.DataException;

/**
 * A dialog for adding a new folder.
 * @author oleg
 */
public class NewChannelDialog {
	private final Shell shell;
	private final Text channelName;
	private final Controller controller;
	/**
	 * Creates a new dialog with the given parent shell and controller.
	 * @param parent parent shell
	 * @param controller controller
	 */
	public NewChannelDialog(Shell parent, final Controller controller) {
		this.controller = controller;
		shell = new Shell(parent,SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText("New folder");
		shell.setLayout(new GridLayout(1,false));
		Composite input = new Composite(shell, SWT.NONE);
		input.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		input.setLayout(new GridLayout(2,false));
		
		Label label = new Label(input,SWT.NONE);
		label.setText("Folder name");
		
		channelName = new Text(input, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.widthHint = 200;
		channelName.setLayoutData(gd);

		channelName.addKeyListener(new KeyAdapter() {
			@Override
			public void keyPressed(KeyEvent event) {
				if (event.keyCode == SWT.CR) addChannel();
			}
		});
		
		Composite buttons = new Composite(shell, SWT.NONE);
		buttons.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		buttons.setLayout(new GridLayout(2, false));
		Button button = new Button(buttons, SWT.NONE);
		button.setText("Create");
		
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				addChannel();
			}
		});
		
		button = new Button(buttons, SWT.NONE);
		button.setText("Cancel");
		
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				shell.close();
			}
		});
		
		shell.pack();
		shell.open();
		while (!shell.isDisposed()) {
			if (!shell.getDisplay().readAndDispatch()) {
				shell.getDisplay().sleep();
			}
		}
	}
	// adds the folder 
	private void addChannel() {
		try {
			controller.getDataManager().addChannel(channelName.getText());
		} catch (DataException e) {
			if (e.isUniquenessViolation()) {
				controller.reportError("A folder with this name already exists", shell);
			} else {
				controller.reportError("Adding new folder caused an error:\n" + e.getMessage(), shell);
			}
			return;
		}
		shell.close();
	}
}
