package view.dialogs;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import controller.Controller;
import data.DataException;
import data.Virtual;
import data.VirtualFactory;

/**
 * Create a new virtual dialog.
 * @author Kuat
 *
 */
public class NewVirtualDialog {
	private final Controller controller;
	private final Shell shell;
	private final Text virtualName;
	private final Virtual virtual;
	
	public NewVirtualDialog(Shell parent, final Controller controller, Virtual v) {		
		this.controller = controller;		
		if (v == null) v = VirtualFactory.empty("");
		this.virtual = v;
		
		shell = new Shell(parent,SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText("New virtual folder");
		shell.setLayout(new GridLayout(1,false));
		Composite input = new Composite(shell, SWT.NONE);
		input.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		input.setLayout(new GridLayout(2,false));
		
		Label label = new Label(input,SWT.NONE);
		label.setText("Folder name");
		
		virtualName = new Text(input, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.widthHint = 200;
		virtualName.setLayoutData(gd);
		
		if (v!= null)
			virtualName.setText(v.getName());
		else 
			virtualName.setText("");
		
		virtualName.selectAll();

		virtualName.addKeyListener(new KeyAdapter() {
			@Override
			public void keyPressed(KeyEvent event) {
				if (event.keyCode == SWT.CR) addVirtual();
			}
		});
		
		Composite buttons = new Composite(shell, SWT.NONE);
		buttons.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		buttons.setLayout(new GridLayout(2, false));
		Button button = new Button(buttons, SWT.NONE);
		button.setText("Create");
		
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				addVirtual();
			}
		});
		
		button = new Button(buttons, SWT.NONE);
		button.setText("Cancel");
		
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				shell.close();
			}
		});
		
		shell.pack();
		shell.open();
		while (!shell.isDisposed()) {
			if (!shell.getDisplay().readAndDispatch()) {
				shell.getDisplay().sleep();
			}
		}
	}
	private boolean addVirtual() {
		try {
			controller.getDataManager().addVirtual(VirtualFactory.changeName(virtual, virtualName.getText()));
		} catch (DataException e) {
			if (e.isUniquenessViolation())
				controller.reportError("A virtual with same name exists", shell);
			else
				controller.reportError("Failed to create a virtual", shell);
			return false;
		}
		shell.close();
		return true;
	}
}
