package view.dialogs;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import controller.Controller;
import data.ArticleContent;
import data.ArticleInfo;
import data.DataException;
 
/**
 * A simple dialog for writing new RSS articles.
 * @author oleg
 */
public class WriteArticleDialog {
	public WriteArticleDialog(final Shell parent, final Controller controller) {
		final Shell shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText("New article");
		shell.setLayout(new GridLayout(1,false));
		Composite input = new Composite(shell, SWT.NONE);
		GridData gd = new GridData(GridData.FILL_BOTH);
		input.setLayoutData(gd);
		input.setLayout(new GridLayout(2, false));
		Label label = new Label(input, SWT.NONE);
		label.setText("Title");
		final Text title = new Text(input, SWT.SINGLE | SWT.BORDER);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.widthHint = 300;
		title.setLayoutData(gd);
		label = new Label(input, SWT.NONE);
		label.setText("Link");
		final Text url = new Text(input, SWT.SINGLE | SWT.BORDER);
		url.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		label = new Label(input, SWT.NONE);
		label.setText("Author");		
		final Text author = new Text(input, SWT.SINGLE | SWT.BORDER);
		author.setText(controller.getUserName());
		author.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		final Text content = new Text(input, SWT.MULTI | SWT.BORDER | SWT.WRAP | SWT.V_SCROLL);
		gd = new GridData(GridData.FILL_BOTH);
		gd.horizontalSpan = 2;
		gd.heightHint = 300;
		content.setLayoutData(gd);
		
		Composite buttons = new Composite(shell, SWT.NONE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		buttons.setLayoutData(gd);
		buttons.setLayout(new GridLayout(2, false));
		
		Button button = new Button(buttons, SWT.NONE);
		button.setText("Save");
		button.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				ArticleInfo info = new ArticleInfo(title.getText(), controller.getDataManager().getFeed("rss-reader://outbox"),System.currentTimeMillis(),author.getText(),url.getText(),true, false);
				ArticleContent summary = new ArticleContent(content.getText());
				try {
					controller.getDataManager().addArticle(info, summary);
				} catch (DataException e) {
					if (e.isUniquenessViolation()) {
						controller.reportError("An article with the same title and date already exists in the outbox", shell);
						return;
					}
				}
				shell.close();
			}
		});
		button = new Button(buttons, SWT.NONE);
		button.setText("Cancel");
		button.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				shell.close();
			}
		});
		shell.pack();
		shell.open();
		
		while (!shell.isDisposed()) {
			if (!parent.getDisplay().readAndDispatch()) {
				parent.getDisplay().sleep();
			}
		}
	}
}
