package view.dnd;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.EOFException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.dnd.ByteArrayTransfer;
import org.eclipse.swt.dnd.TransferData;

import data.DNDArticle;


/**
 * A byte array DND transfer for DNDArticle.
 * @author oleg
 */
public class ArticleTransfer extends ByteArrayTransfer {
	private static final String articleTransferName = "ArticleTransfer";
	private static final int articleTransferId = registerType(articleTransferName);
	private static ArticleTransfer instance = new ArticleTransfer();
	
	/**
	 * The instance of the transfer class.
	 * @return the instance of the transfer class
	 */
	public static ArticleTransfer instance() {
		return instance;
	}
	public void javaToNative (Object object, TransferData transferData) {
	 	if (object == null || !(object instanceof DNDArticle[])) {
	 		return;
	 	}
	 	
	 	if (isSupportedType(transferData)) {
	 		DNDArticle[] infos = (DNDArticle[]) object;	
	 		try {
	 			ByteArrayOutputStream out = new ByteArrayOutputStream();
	 			ObjectOutputStream oo = new ObjectOutputStream(out);
	 			for (DNDArticle info: infos) {
	 				oo.writeObject(info);
	 			}
	 			byte[] buffer = out.toByteArray();
	 			oo.close();
	 
	 			super.javaToNative(buffer, transferData);
	 			
	 		} catch (IOException e) {
	 			e.printStackTrace();
	 		}
	 	}
	 }
	 public Object nativeToJava(TransferData transferData){	
	 
	 	if (isSupportedType(transferData)) {
	 		
	 		byte[] buffer = (byte[])super.nativeToJava(transferData);
	 		if (buffer == null) return null;
	 		
	 		List<DNDArticle> infos = new ArrayList<DNDArticle>();
	 		try {
	 			ByteArrayInputStream in = new ByteArrayInputStream(buffer);
	 			ObjectInputStream oi = new ObjectInputStream(in);
	 			for (;;) {
	 				try {
	 					infos.add((DNDArticle) oi.readObject());
	 				} catch (EOFException e) {
	 					break;
	 				}
	 			}
	 			oi.close();
	 		} catch (IOException ex) {
	 			return null;
	 		} catch (ClassNotFoundException ex) {
	 			return null;
	 		}
	 		return infos.toArray();
	 	}
	 
	 	return null;
	 }
	
	
	@Override
	protected int[] getTypeIds() {
		return new int[]{articleTransferId};
	}

	@Override
	protected String[] getTypeNames() {
		return new String[]{articleTransferName};
	}
	
}
