package webapi;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;

import org.apache.commons.lang.StringUtils;

import util.Trace;
import controller.GlobalPreferences;
import data.GeoPoint;

/**
 * Communicates to Google Maps geocoder.
 * @author Kuat
 *
 */
public class GeoCoder {

	/**
	 * Requests the geographical coordinates of an address. If the location is not found
	 * it return INFINITY.
	 * @param address
	 * @return GeoPoint which is a pair of latitude and longitude 
	 * @throws WebConnectionException if a connection error occurs
	 */
	public static final GeoPoint getLocation(String address) throws WebConnectionException {
		try	{
			address = address.trim().replace(" ", "+");
			String geoQuery = new StringBuffer("http://maps.google.com/maps/geo?q=").append(address).append("&output=csv&key=").append(GlobalPreferences.GOOGLE_KEY).toString();

			BufferedReader br = new BufferedReader(new InputStreamReader(new URL(geoQuery).openStream())); 
			String line = br.readLine();
			br.close();
			
			if (line == null) return null;
			
			String[] array = StringUtils.splitByWholeSeparator(line, ",");
			double latitude = Double.parseDouble(array[2]);
			double longitude = Double.parseDouble(array[3]);	
			
			if ((latitude == 0) && (longitude == 0)) return GeoPoint.INFINITY;
			
			return new GeoPoint(latitude, longitude);
			
		} catch (IOException e) {
			Trace.printVerbose("Error: " + e.getMessage());
			throw new WebConnectionException("Could not connect to the Google service");
		}
	}

}
