package webapi;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.StringTokenizer;

import util.Pair;

/**
 * Commmunicates with a NetGeo server.
 * @author oleg
 */
public class NetGeo {
	private static String urlToIP(String url) throws WebConnectionException {
		try {
			return InetAddress.getByName(new URL(url).getHost()).getHostAddress();
		} catch (UnknownHostException e) {
			throw new WebConnectionException("Unable to resolve the IP address of the host of the feed " + url);
		} catch (MalformedURLException e) {
			throw new WebConnectionException("Feed URL is invalid " + url);
		}
	}
	/**
	 * Requests the geographical coordinates of a URL.
	 * @param url the URL
	 * @return a pair of latitude and longitude
	 * @throws WebConnectionException if a connection error occurs
	 */
	public static final Pair<String, String> getLocation(String url) throws WebConnectionException {
		try {
			String ip = urlToIP(url); 
			String netGeoQuery = new StringBuffer("http://netgeo.caida.org/perl/netgeo.cgi?target=").append(ip).append("&").append("method=getLatLong").toString();
			BufferedReader br = new BufferedReader(new InputStreamReader(new URL(netGeoQuery).openStream())); 
			String latitude = null;
			String longitude = null;
			String line = br.readLine();
			while (line != null) {	
				StringTokenizer st = new StringTokenizer(line, "\t <");
				if (st.hasMoreTokens()) {
					String prop = st.nextToken();
					if (prop.equals("LAT:") && st.hasMoreTokens()) latitude = st.nextToken();
					else if (prop.equals("LONG:") && st.hasMoreTokens()) longitude = st.nextToken();
				}
				line = br.readLine();
			}
			br.close();
			if (latitude == null || longitude == null) return null;
			else return new Pair<String, String>(latitude, longitude);
		} catch (IOException e) {
			throw new WebConnectionException("Could not connect to the NetGeo service");
		}

	}

}
