package webapi.search;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.swt.graphics.ImageData;
import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import util.IconFactory;
import util.IconFactory.ICON;
import webapi.WebConnectionException;

/**
 * BlogDigger search support.
 * @author oleg
 */
public class BlogDiggerSearch implements WebSearchProvider {

	private static final String NAME = "BlogDigger";
	private static final String BASE = "http://www.blogdigger.com/search?sortby=date&type=rss&pp=10";
	
	public ImageData getIcon() {
		return IconFactory.getIcon(ICON.SEARCH_BLOGDIGGER);
	}

	public String getName() {
		return NAME;
	}

	public Collection<WebSearchResult> searchWeb(String query, int maxresult) throws WebConnectionException {
		SAXBuilder builder = new SAXBuilder();
		try {
			List<WebSearchResult> results = new LinkedList<WebSearchResult>();
			int remain = maxresult;
			int offset = 0;
			while (remain > 0) {
				String url = new StringBuffer(BASE).append("&q=").append(query).append("&si=").append(offset + 1).toString();
				offset += 10;
				Document xml = builder.build(new URL(url));
				Element channel = xml.getRootElement().getChild("channel");
				if (channel != null) {
					Iterator iter = channel.getChildren("item").iterator();
					if (!iter.hasNext()) break;
					while (iter.hasNext() && remain > 0) {
						Element item = (Element) iter.next();
						Element source = item.getChild("source");
						if (source != null) {
							Attribute urlattr = source.getAttribute("url");
							if (urlattr != null) {
								String siteURL = "";
								Element elt = item.getChild("link");
								if (elt != null) siteURL = elt.getValue();
								String info = "";
								elt = item.getChild("description");
								if (elt != null) info = elt.getValue();
								results.add(new WebSearchResult(source.getValue(),siteURL,urlattr.getValue(),info));
								remain--;
							}
						}
					}
				}
			}
			return results;
		} catch (MalformedURLException e) {
			throw new WebConnectionException(NAME + " search error: malformed query");
		} catch (JDOMException e) {
			throw new WebConnectionException(NAME + " search error: could not parse the response");
		} catch (IOException e) {
			throw new WebConnectionException("Failed to connect to the " + NAME + " service");
		}
	}
	
}
