package webapi.search;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.swt.graphics.ImageData;
import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.input.SAXBuilder;

import util.IconFactory;
import webapi.WebConnectionException;
/**
 * Searches Feedster for feeds.
 * @author oleg
 */
public class FeedsterSearch implements WebSearchProvider {
	private final String NAME = "Feedster";
	private final String BASE = "http://www.feedster.com/search/type/rss/";
	
	public ImageData getIcon() {
		return IconFactory.getIcon(IconFactory.ICON.SEARCH_FEEDSTER);
	}

	public String getName() {
		return NAME;
	}

	public Collection<WebSearchResult> searchWeb(String query, int maxresult) throws WebConnectionException {
		SAXBuilder builder = new SAXBuilder();
		try {
			List<WebSearchResult> results = new LinkedList<WebSearchResult>();
			int remain = maxresult;
			String pageID = "";
			int offset = 0;
			while (remain > 0) {
				if (pageID == null) break;
				String url;
				if (offset == 0) url = BASE + query;
				else url = new StringBuffer(BASE).append(pageID).append("/offset/").append(offset).append("/").append(query).toString();
				offset += 10;
				Document xml = builder.build(new URL(url));
				Namespace feedsterNamespace = Namespace.getNamespace("feedstersearch", "http://feedster.com/feedstersearch/ext/1.0");
				Element channel = xml.getRootElement().getChild("channel");
				if (channel != null) {
					Element elt = channel.getChild("pageId", feedsterNamespace); 
					if (elt == null) pageID = null;
					else pageID = elt.getValue();
					Iterator iter = channel.getChildren("item").iterator();
					if (!iter.hasNext()) break;
					while (iter.hasNext() && remain > 0) {
						Element item = (Element) iter.next();
						Element source = item.getChild("source");
						if (source != null) {
							Attribute urlattr = source.getAttribute("url");
							if (urlattr != null) {
								String siteURL = "";
								elt = item.getChild("link");
								if (elt != null) siteURL = elt.getValue();
								String info = "";
								elt = item.getChild("description");
								if (elt != null) info = elt.getValue();
								results.add(new WebSearchResult(source.getValue(),siteURL,urlattr.getValue(),info));
								remain--;
							}
						}
					}
				}
			}
			return results;
		} catch (MalformedURLException e) {
			throw new WebConnectionException(NAME + " search error: malformed query");
		} catch (JDOMException e) {
			throw new WebConnectionException(NAME + " search error: could not parse the response");
		} catch (IOException e) {
			throw new WebConnectionException("Failed to connect to the " + NAME + " service");
		}
	}

}
