package webapi.search;

import java.util.Collection;

import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.LocationEvent;
import org.eclipse.swt.browser.LocationListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Composite;

import util.Pair;
import view.dialogs.SubscribeDialog;
import webapi.GoogleMapsWidget;
import webapi.NetGeo;
import webapi.WebConnectionException;
import webapi.GoogleMapsWidget.ZoomControl;
import controller.Controller;

/**
 * Displays feed search results on a Google Map.
 * @author oleg
 */
public class GoogleMapsSearchView extends WebSearchView {
	final GoogleMapsWidget mapPane;

	private static final int INFO_LENGTH_LIMIT = 300;
	
	private String limitString(String s, int limit) {
		if (s.length() > limit) {
			return s.substring(0, limit) + "...";
		} else return s;
	}

	
	// converts feed info into an html snippet
	private String infoHTML(WebSearchResult result) {
		// Kuat: prevents overflow of the balloon
		//	StringBuffer info = new StringBuffer("<div><a style=\"font: 10pt Tahoma\" href=\"follow://");
		StringBuffer info = new StringBuffer("<div style=\"width: 14em;\"><a style=\"font: 10pt Tahoma\" href=\"follow://");
		info.append(result.getSiteURL()).append("\">").append(result.getTitle()).append("</a>&nbsp;<a style=\"font: 8pt Tahoma\" href=\"").append("addfeed://").append(result.getFeedURL()).append("\">").append("add</a>").append("<p style=\"font: 8pt Tahoma\">").append(limitString(result.getInfo(),INFO_LENGTH_LIMIT)).append("</p></div>");
		return info.toString();
	}

	/**
	 * Creates a new search view.
	 * @param parent parent pane
	 * @param ctrl controller
	 */
	public GoogleMapsSearchView(Composite parent, Controller ctrl) {
		super(parent, ctrl);
		setLayout(new FillLayout());
		mapPane = new GoogleMapsWidget(this, SWT.NONE, ZoomControl.SWT_SCALE, null);
		mapPane.addLocationListener(new LocationListener() {

			public void changed(LocationEvent event) {

			}

			public void changing(LocationEvent event) {
				if (!event.location.startsWith("file://")) {
					if (event.location.startsWith("addfeed://")) {
						event.doit = false;
						new SubscribeDialog(getShell(), controller, event.location.substring("addfeed://".length()),null,"");
					} else if (event.location.startsWith("follow://")) {
						event.doit = false;
						Program.launch(event.location.substring("follow://".length()));
					}
				}

			}

		});

	}
	/**
	 * Displays search results.
	 */
	public void show(final Collection<WebSearchResult> results) {
		if (!this.isDisposed()) {
			this.getDisplay().asyncExec(new Runnable() {
				public void run() {
					mapPane.clearMarkers();
					for (WebSearchResult result: results) {
						try {
							Pair<String, String> location = NetGeo.getLocation(result.getFeedURL());
							if (location != null) mapPane.addMarker(location.car, location.cdr, infoHTML(result),true);
						} catch (WebConnectionException e) {
							e.printStackTrace();
						}
					}
				}
			});
		}
	}


}
