package webapi.search;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Collection;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

import org.apache.xmlrpc.XmlRpcClient;
import org.apache.xmlrpc.XmlRpcException;
import org.eclipse.swt.graphics.ImageData;

import util.IconFactory;
import webapi.WebConnectionException;

/**
 * Searches Syndic8 for feeds.
 * @author oleg
 */
public class Syndic8Search implements WebSearchProvider {
	private final String NAME = "Syndic8";
	public ImageData getIcon() {
		return IconFactory.getIcon(IconFactory.ICON.SEARCH_SYNDIC8);
	}

	@SuppressWarnings("unchecked")
	public Collection<WebSearchResult> searchWeb(String query, int maxresult) throws WebConnectionException {
		List<WebSearchResult> results = new LinkedList<WebSearchResult>();
		try {
			XmlRpcClient client = new XmlRpcClient("http://www.syndic8.com/xmlrpc.php");
			Vector arg = new Vector();
			arg.add(query);
			arg.add("date_created");
			arg.add(maxresult);
			Object result = client.execute("syndic8.FindFeeds",arg);
			arg.clear();
			arg.add(result);
			Object feedInfos = client.execute("syndic8.GetFeedInfo", arg);
			for (Object o: (Vector) feedInfos) {
				Hashtable info = (Hashtable) o;
				String title = (String) info.get("sitename");
				if (title == null) title = "";
				String siteURL = (String) info.get("siteurl");
				if (siteURL == null) siteURL = "";
				String feedURL = (String) info.get("dataurl");
				if (feedURL == null) feedURL = "";
				String description = (String) info.get("description");
				if (description == null) description = "";
				results.add(new WebSearchResult(title,siteURL,feedURL,description));
			}
			return  results;
		} catch (MalformedURLException e) {
			throw new WebConnectionException(NAME + " search error: malformed query");
		} catch (XmlRpcException e) {
			throw new WebConnectionException(NAME + " search error: could not parse the response");
		} catch (IOException e) {
			throw new WebConnectionException("Failed to connect to the " + NAME + " service");
		}
	}


	public String getName() {
		return NAME;
	}

}
