package webapi.search;

import java.util.Collection;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

import view.dialogs.SubscribeDialog;
import controller.Controller;

/**
 * Displays the results of a web search as a list.
 * @author oleg
 */
public class TableSearchView extends WebSearchView {
	private final Table table;
	private WebSearchResult[] memory = new WebSearchResult[] {};
	private final TableEditor editor;
	private final Button addButton;
	/**
	 * Creates a new search view on a parent pane.
	 * @param parent parent pane.
	 * @param ctrl controller
	 */
	public TableSearchView(Composite parent, Controller ctrl) {
		super(parent, ctrl);
		this.setLayout(new FillLayout());
		table = new Table(this, SWT.FULL_SELECTION | SWT.HIDE_SELECTION | SWT.SINGLE | SWT.VIRTUAL);
		new TableColumn(table, SWT.NONE).setWidth(500);
		table.addListener(SWT.SetData, new Listener() {
			public void handleEvent(Event event) {
				((TableItem) event.item).setText(memory[event.index].getTitle());
			}
		});

		editor = new TableEditor(table);
		editor.horizontalAlignment = SWT.RIGHT;
		editor.grabVertical = true;
		editor.minimumWidth = 50;
		
		addButton = new Button(table, SWT.NONE);
		addButton.setText("add");
		
		addButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				new SubscribeDialog(getShell(),controller,memory[table.getSelectionIndex()].getFeedURL(),null,memory[table.getSelectionIndex()].getTitle());
			}
		});
		
		FontData fd = addButton.getFont().getFontData()[0];
		fd.setHeight(fd.getHeight() - 2);
		addButton.setFont(new Font(getDisplay(),fd));
		
		table.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				editor.setEditor(addButton, (TableItem) event.item, 0);
			}
		});
	}
	
	@Override
	public void show(final Collection<WebSearchResult> results) {
		getDisplay().asyncExec(new Runnable()  {
			public void run() {
				table.setItemCount(results.size());
				memory = (WebSearchResult[]) results.toArray(memory);
				table.clearAll();
			}
		});
	}
	
}
