package webapi.search;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.swt.graphics.ImageData;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import util.IconFactory;
import webapi.WebConnectionException;
/**
 * Searches Technorati for feeds.
 * @author oleg
 */
public class TechnoratiSearch implements WebSearchProvider {
	private static final String NAME = "Technorati";
	private static final String KEY = "748d0b8425a01a669620005475c0c3a2";
	public ImageData getIcon() {
		return IconFactory.getIcon(IconFactory.ICON.SEARCH_TECHNORATI);
	}

	public String getName() {
		return NAME;
	}

	public Collection<WebSearchResult> searchWeb(String query, int maxresult) throws WebConnectionException {
		SAXBuilder builder = new SAXBuilder();
		try {
			Document xml = builder.build(new URL(new StringBuffer("http://api.technorati.com/search?key=").append(KEY).append("&query=").append(query).append("&limit=").append(maxresult).toString()));
			Element elt = xml.getRootElement().getChild("document");
			List<WebSearchResult> results = new LinkedList<WebSearchResult>();
			if (elt != null) {
				List items  = elt.getChildren("item");
				for (Object item: items) {
					Element weblog = ((Element) item).getChild("weblog");
					if (weblog != null) {
						Element feed = weblog.getChild("atomurl");
						if (feed == null || feed.getValue().equals("")) feed = weblog.getChild("rssurl");
						if (feed != null) {
							String title = "";
							elt = weblog.getChild("name");
							if (elt != null) title = elt.getValue();
							String siteUrl = "";
							elt = weblog.getChild("url");
							if (elt != null) siteUrl = elt.getValue();
							String info = "";
							elt = ((Element) item).getChild("excerpt");
							info = elt.getValue();
							results.add(new WebSearchResult(title, siteUrl,feed.getValue(),info));
						}
					}
				}
			}
			return results;
		} catch (MalformedURLException e) {
			throw new WebConnectionException(NAME + " search error: malformed query");
		} catch (JDOMException e) {
			throw new WebConnectionException(NAME + " search error: could not parse the response");
		} catch (IOException e) {
			throw new WebConnectionException("Failed to connect to the " + NAME + " service");
		}

	}

}
