package webapi.search;

import java.util.Collection;

import org.eclipse.swt.graphics.ImageData;

import webapi.WebConnectionException;

/**
 * Interface of a web feed search engine.
 * @author oleg
 */
public interface WebSearchProvider {
	/**
	 * Searches the web for feeds.
	 * @param query the query
	 * @param maxresult the maximum number of results
	 * @return a collection of at most <code>maxresult</code> feeds found
	 * @throws WebConnectionException if a connection/authentication error occurs
	 */
	public Collection<WebSearchResult> searchWeb(String query, int maxresult) throws WebConnectionException;
	/**
	 * Gets the name of the search engine.
	 * @return the name of the engine
	 */
	public String getName();
	/**
	 * Gets an image associated with the search engine.
	 * @return an image
	 */
	public ImageData getIcon();
}
